import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
import csv
from sscws.sscws import SscWs
from datetime import datetime,timedelta
from sys import argv

def main(run,ts,te,sc):
    ssc=SscWs()

    result=ssc.get_locations([sc],[ts,te])
    data=result['Data'][0]
    coords=data['Coordinates'][0]
    df=pd.DataFrame(coords)
    df['t']=data['Time']
    df['formatted_datetime']=df.t.dt.strftime('%y/%m/%d %H:%M:%S')
    df2=df[['formatted_datetime','X','Y','Z']]
    df2.loc[:,'X']/=6378.16
    df2.loc[:,'Y']/=6378.16
    df2.loc[:,'Z']/=6378.16

    df2.to_csv('test/orbit.txt',header=False,index=False,sep='\t')
    return datetime,timedelta

if __name__=='__main__':
    main(*argv[1:])



import os, locale
import numpy as np
import matplotlib.pyplot as plt
#from mpl_toolkits.basemap import cm
from math import log10


def mhd_vpkg(val):
  "set colormap, title, and range of variables for mhd plot"

  print (val,val[:3])

#######
  if   val[:2]=="bx":
    cmap=plt.cm.bwr				# name of colormap
    name='Bx [nT]'				# name for colobar title
    vran=[-30.0,30.0]				# default range of value
    nlev=51 					# number of coutour level
    tick=10.					# interval of major ticks for colorbar
    ufac=1.	  				# unit factor for a val in a desried unit
    norm=0					# 0 for linear-scale, 1 for log-scale

  elif val[:2]=="by":
    cmap=plt.cm.bwr
    name='By [nT]'
    vran=[-30.0,30.0]
    nlev=51
    tick=10.
    ufac=1.
    norm=0

  elif val[:2]=="bz":
    cmap=plt.cm.bwr
    name='Bz [nT]'
    vran=[-30.0,30.0]
    nlev=51
    tick=10.
    ufac=1.
    norm=0

  elif val[:2]=="bt":
    cmap=plt.cm.jet
    name='|B| [nT]'
    vran=[0.0,30.0]
    nlev=51
    tick=10.
    ufac=1.
    norm=0

#######
  elif val=="vx":
    cmap=plt.cm.bwr                           # name of colormap
    name='Vx [km/s]'                            # name for colobar title
    vran=[-300.0,300.0]                         # default range of value
    nlev=51                                     # number of coutour level
    tick=100.                                   # interval of major ticks for colorbar
    ufac=1.                                     # unit factor for a val in a desried unit
    norm=0                                      # 0 for linear-scale, 1 for log-scale

  elif val=="vy":
    cmap=plt.cm.bwr
    name='Vy [km/s]'
    vran=[-100.0,100.0]
    nlev=51
    tick=25.
    ufac=1.
    norm=0

  elif val=="vz":
    cmap=plt.cm.bwr
    name='Vz [km/s]'
    vran=[-100.0,100.0]
    nlev=51
    tick=25.
    ufac=1.
    norm=0

  elif val=="vt":
    cmap=plt.cm.jet
    name='|V| [km/s]'
    vran=[0.0,400.0]
    nlev=51
    tick=50.
    ufac=1.
    norm=0



#######
  elif val=="jx":
    cmap=plt.cm.bwr                           # name of colormap
    name='Jx [nA/m$^2$]'                        # name for colobar title
    vran=[-10.0,10.0]                             # default range of value
    nlev=51                                     # number of coutour level
    tick=2.                                     # interval of major ticks for colorbar
    ufac=1.e3                                   # unit factor for a val in a desried unit
    norm=0                                      # 0 for linear-scale, 1 for log-scale

  elif val=="jy":
    cmap=plt.cm.bwr
    name='Jy [nA/m$^2$]'
    vran=[-10.0,10.0]
    nlev=51
    tick=2.
    ufac=1.e3
    norm=0

  elif val=="jz":
    cmap=plt.cm.bwr
    name='Jz [nA/m$^2$]'
    vran=[-10.0,10.0]
    nlev=51
    tick=2.
    ufac=1.e3
    norm=0

  elif val=="jt":
    cmap=plt.cm.jet
    name='|J| [nA/m$^2$]'
    vran=[0.0,10.0]
    nlev=51
    tick=2.
    ufac=1.e3
    norm=0



#######
  elif val[:2]=="ex":
    cmap=plt.cm.bwr                         # name of colormap
    name='Ex [mV/m]'                            # name for colobar title
    vran=[-3.0,3.0]                           # default range of value
    nlev=51                                     # number of coutour level
    tick=1.                                    # interval of major ticks for colorbar
    ufac=1.                                     # unit factor for a val in a desried unit
    norm=0                                      # 0 for linear-scale, 1 for log-scale

  elif val[:2]=="ey":
    cmap=plt.cm.bwr
    name='Ey [mV/m]'
    vran=[-3.0,3.0]
    nlev=51
    tick=1.
    ufac=1.
    norm=0

  elif val[:2]=="ez":
    cmap=plt.cm.bwr
    name='Ez [mV/m]'
    vran=[-3.0,3.0]
    nlev=51
    tick=1.
    ufac=1.
    norm=0

  elif val[:2]=="et":
    cmap=plt.cm.jet
    name='|E| [mV/m]'
    vran=[0.0,5.0]
    nlev=51
    tick=1.
    ufac=1.
    norm=0

  elif val[:4]=="epar":
    cmap=plt.cm.bwr
    name='Epar [mV/m]'
    vran=[-1.0,1.0]
    nlev=51
    tick=0.5
    ufac=1.
    norm=0

  elif val[:4]=="eper":
    cmap=plt.cm.bwr
    name='Eper [mV/m]'
    vran=[-2.0,2.0]
    nlev=51
    tick=1.
    ufac=1.
    norm=0


#######
  elif val=="rr":
    cmap=plt.cm.jet                             # name of colormap
#   cmap=plt.cm.nipy_spectral
    name='N [cm$^{-3}$]'                        # name for colobar title
    vran=[0.0,40.0]                             # default range of value
#   vran=[0.0,80.0]                             # default range of value
    nlev=51                                     # number of coutour level
    tick=10.                                    # interval of major ticks for colorbar
    ufac=1.                                     # unit factor for a val in a desried unit
    norm=0                                      # 0 for linear-scale, 1 for log-scale

  elif val=="pp":
    cmap=plt.cm.jet
    name='Pp [pPa]'
    vran=[0.0,3000.0]
    nlev=51
    tick=500.
    ufac=1.
    norm=0

  elif val=="pdy":
    cmap=plt.cm.jet
    name='Pdy [nPa]'
    vran=[0.0,20.0]
    nlev=51
    tick=5.
    ufac=1.
    norm=0

  elif val=="kev":
    cmap=plt.cm.jet
    name='T [keV]'
    vran=[0,1.0]
    nlev=51
    tick=0.2
    ufac=1.
    norm=0


#######
  elif val=="resis":
    cmap=plt.cm.jet
    name='Resis [Ohm m]'
    vran=[0,0.5]
    nlev=51
    tick=0.1
    ufac=1.
    norm=0


  elif val=="absepar":
    cmap=plt.cm.jet
    name='absepar [V/m]'
    vran=[1e-7,1e-1]
    nlev=51
    tick=0.1
    ufac=1.
    norm=1

  elif val=="eint":
    cmap=plt.cm.bwr
    name=r'$\int E_{||} ds$ [kV]'
    vran=[-30.0,30.0]
    nlev=51
    tick=10.
    ufac=1.
    norm=0

  elif val=="abseint":
    cmap=plt.cm.jet
    name=r'$| \int E_{||} ds |$ [kV]'
    vran=[0.0,30.0]
    nlev=51
    tick=5.
    ufac=1.
    norm=0

  elif val=="fbalx":
    cmap=plt.cm.bwr				# name of colormap
    name='fbalx [fN/m$^3$]'                     # name for colobar title
    vran=[-50.0,50.0]                           # default range of value
    nlev=51                                     # number of coutour level
    tick=10.                                    # interval of major ticks for colorbar
    ufac=1.                                     # unit factor for a val in a desried unit
    norm=0                                      # 0 for linear-scale, 1 for log-scale

  elif val=="fbaly":
    cmap=plt.cm.bwr
    name='fbaly [fN/m$^3$]'
    vran=[-50.0,50.0]
    nlev=51
    tick=10.
    ufac=1.
    norm=0

  elif val=="zgse":
    cmap=plt.cm.bwr
    name='Zgse [Re]'
    vran=[-20.,20.]
    nlev=51
    tick=5.
    ufac=1.
    norm=0


####### xray values
  elif val=='xray':
    cmap=plt.cm.jet
    name='Xray Rate [keVcm$^{-2}$s$^{-1}$sr$^{-1}$]'
    vran=[0.,20.]
    nlev=51
    tick=5.
    ufac=1e-3
    norm=0

  elif val=='xraycut':
    cmap=plt.cm.jet
    name='Xray Rate [keVcm$^{-3}$s$^{-1}$]'
    vran=[0.,3.]
    nlev=51
    tick=0.5
    ufac=1e-3
    norm=0

  elif val=='Qcut':
    cmap=plt.cm.jet
    name='Qcut [LOG cm$^{-5}$s$^{-1}$]'
    vran=[1e8,1e12]
#   vran=[1e7,1e12]
    nlev=51
    tick=10.
    ufac=1.
    norm=1


  elif val=='nsw':
    cmap=plt.cm.jet
    name='Nsw [cm$^{-3}$]'
    vran=[0.,50.]
    nlev=51
    tick=10
    ufac=1.
    norm=0

  elif val=='nn':
    cmap=plt.cm.jet
    name='Nn [cm$^{-3}$]'
    vran=[0.,20.]
    nlev=51
    tick=5
    ufac=1.
    norm=0

  elif val=='vsw':
    cmap=plt.cm.jet
    name='Vsw [km/s]'
    vran=[0.,400.]
    nlev=51
    tick=50
    ufac=1.
    norm=0

  elif val=='vth':
    cmap=plt.cm.jet
    name='Vth [km/s]'
    vran=[0.,800.]
    nlev=51
    tick=100
    ufac=1.
    norm=0

  elif val=='vav':
    cmap=plt.cm.jet
    name='<V> [km/s]'
    vran=[0.,800.]
    nlev=51
    tick=100
    ufac=1.
    norm=0

  elif val=='tev':
    cmap=plt.cm.jet
    name='T [eV]'
    vran=[0.,400.]
    nlev=51
    tick=50
    ufac=1.
    norm=0


  else  :		# defalut values
    cmap=plt.cm.jet
    name=val+' [?]'
    vran=[0,100.0]
    nlev=51
    tick=20.
    ufac=1.
    norm=0


  if norm==0:   # for linear-scale contour
     dlev = (vran[1]-vran[0])/(nlev-1)		 	# interval of contour level
     clev = np.arange(vran[0],vran[1]+dlev,dlev)	# contour levels
     tlev = np.arange(vran[0],vran[1]+tick,tick) 	# tick levels for the colorbar

  if norm==1: 	# for log-scale contour
     t1=log10(vran[0])
     t2=log10(vran[1])
     dt=log10(tick)

     dlev=(t2-t1)/(nlev-1)
     clev=[ 10**i for i in np.arange(t1,t2+dlev,dlev)]
     tlev=[ 10**i for i in np.arange(t1,t2+dt,dt) ]


  pkg_plot=[name,cmap,ufac,clev,tlev,norm]

  return pkg_plot



import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
from sys import argv
from Msh_Nstep_test_n import main as mhd
from Msh_Nstep_Soucek import main as Soucek
from Msh_Nstep_Romashets import main as Romashets
from Msh_Nstep_Spreiter_test import main as Spreiter

def main(run, sc,mpdo=0,bsdo=0):
    """PURPOSE: Generate models result for given SW/IMF conditions & virtual spacecraft trace

    CALL: python Msh_Nstep_master.py run_name sc_name mpoff bsoff

    ARGUMENTS:
    1. SW/IMF conditions file
    text format: (without header)
    Year DOY HR MN Bx By Bz Vx Vy Vz n Pd Ma Mm, as in OMNIweb
    time is for the matching with sc trace.
    One line data can be used for constant SW cond.

    2. sc trace file
    text format: (without header)
    yy/mm/dd yy/mm/dd x y z, as in SSCWeb (x,y,z in GSE)

    3. (Optional) magnetopause offset
    Manual magnetopause offset along x axis
    Magnetopause model for the MHD-based MSH model is Shue et al. 1998.

    4. (Optional) bow shock offset
    Manual bow shock offset along x axis
    Bow shock model for the MHD-based MSH model is Jerab et al. 2005.


    """

    f_sw='/Volumes/easystore/openggcm_run/Msh_Nstep/case_studies/'+run+'/omni.lst'
    #SW/IMF conditions file
    xyz='/Volumes/easystore/openggcm_run/Msh_Nstep/case_studies/'+run+'/'+sc+'_orbit.txt'
    #sc trace file
    fout='/Volumes/easystore/openggcm_run/Msh_Nstep/case_studies/'+run+'/Msh_test_out.txt'
    #MHD-based model output file
    fout_v='/Volumes/easystore/openggcm_run/Msh_Nstep/case_studies/'+run+'/Msh_Soucek_out.txt'
    #Soucek model velocity output file
    fout_b='/Volumes/easystore/openggcm_run/Msh_Nstep/case_studies/'+run+'/Msh_Romashets_out.txt'
    #Romashets model magnetic field output file
    fout_n='/Volumes/easystore/openggcm_run/Msh_Nstep/case_studies/'+run+'/Msh_Spreiter_out.txt'
    #Spreiter plasma density & temperature output file
    foff='/Volumes/easystore/openggcm_run/Msh_Nstep/case_studies/'+run+'/Msh_offset.txt'

    mhd(xyz,f_sw,fout,mpdo,bsdo,foff)
    print('MHD-based model result calculated\n')
    #Spreiter(xyz,f_sw,fout_n,mpdo,bsdo)
    print('Spreiter model n,T result calculated\n')
    #Soucek(xyz,f_sw,fout_v,mpdo,bsdo)
    print('Soucek & Escoubet 2012 velocity model result calculated\n')
    #Romashets(xyz,f_sw,fout_b,mpdo,bsdo)
    print('Romashets & Vandas 2019 magnetic field result calculated\n')
    return

if __name__=='__main__':
    main(*argv[1:])



import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import pickle as pl
import io
from read_mhd                import *
from pkg_mhdplot             import mhd_vpkg
from openggcm_info           import runinfo,mhdtime,mhdt2tstr
from matplotlib.colors       import LogNorm
from matplotlib.ticker       import MultipleLocator, FuncFormatter
from mpl_toolkits.axes_grid1 import make_axes_locatable

def mhd_2Dplot(nrow,ncol,fps,pn,lp,xran,yran,xx,yy,val,vstr,title,mpbs):
  "plot mhd"

####### tick info for pn=xr
  def xr_zgse(x, pos):
    value=x*cos(radians(lp))
    return '%2d' % (round(value))
  def xr_ygse(x, pos):
    value=-x*sin(radians(lp))
    return '%2d' % (round(value))

####### draw graphs
  fig = plt.figure(figsize=(8,11))

  for i in range(len(val)):

    ax1      = fig.add_subplot(nrow,ncol+1,2*i+1,aspect='equal')
    ax1.tick_params(axis='both',which='major',labelsize=7)
    x,y      = np.meshgrid(xx,yy)
    pkg_plot = mhd_vpkg(vstr[i])

    if pkg_plot[5]==1:
      cs    = ax1.contourf(x,y,val[i]*pkg_plot[2],levels=pkg_plot[3], \
              cmap=pkg_plot[1],norm=LogNorm())
    else :
#      print(np.shape(x),np.shape(y),np.shape(val[i]),np.shape(pkg_plot[2]))
      cs    = ax1.contourf(x,y,val[i]*pkg_plot[2],levels=pkg_plot[3], cmap=pkg_plot[1],extend='both')
      if 'y' in mpbs:
          cs2   = ax1.plot(mpbs.xmp,mpbs.y)
          cs2   = ax1.plot(mpbs.xbs,mpbs.y)
      if 'z' in mpbs:
          cs2   = ax1.plot(mpbs.xmp,mpbs.z)
          cs2   = ax1.plot(mpbs.xbs,mpbs.z)

    cax1  = make_axes_locatable(ax1)
    cax2  = cax1.append_axes("right",size="3%",pad="5%")
    cb    = fig.colorbar(cs,cax=cax2,ticks=pkg_plot[4])
    cl    = plt.getp(cb.ax, 'ymajorticklabels')
    plt.setp(cl, fontsize=7)


#   circle1=plt.Circle((2.,0),3,facecolor='None',edgecolor='magenta')
#   circle2=plt.Circle((-2.,0),3,facecolor='None',edgecolor='magenta')
#   ax1.add_artist(circle1)
#   ax1.add_artist(circle2)


####### tick infomation

    if pn!='xr':
      ax1.yaxis.set_major_locator(MultipleLocator(yran[2]))
      ax1.yaxis.set_minor_locator(MultipleLocator(yran[3]))
      if pn=='xy' : xl='Xgse [Re]';yl='Ygse [Re]'
      if pn=='xz' : xl='Xgse [Re]';yl='Zgse [Re]'
      if pn=='yz' : xl='Ygse [Re]';yl='Zgse [Re]'
      if pn=='sc' : xl='Spin angle [deg]';yl='Polar angle [deg]'

    if pn=='xr': # xaxis rotation
      xl      = 'Xgse [Re]'
      yl      = 'Zgse [Re]'

    if pn=='zr': # zaxis rotation
      xl      = 'Xgse [Re]'
      yl      = 'Zgse [Re]'
## calculate zgse
      ax1.yaxis.set_major_locator(MultipleLocator(abs(yran[2]/np.cos(np.radians(lp)))))
      ax1.yaxis.set_minor_locator(MultipleLocator(abs(yran[3]/np.cos(np.radians(lp)))))
      ax1.yaxis.set_major_formatter(FuncFormatter(xr_zgse))
## calculate ygse
#     aa1=ax1.twinx()
#     aa1.set_ylim(-yran[0]*np.sin(np.radians(lp)),-yran[1]*np.sin(np.radians(lp)))
#     aa1.yaxis.set_major_locator(MultipleLocator(abs(yran[2]/np.sin(np.radians(lp)))))
#     aa1.yaxis.set_minor_locator(MultipleLocator(abs(yran[3]/np.sin(np.radians(lp)))))
#     aa1.grid(True)
#     aa1.yaxis.set_major_formatter(FuncFormatter(xr_ygse))

    ax1.xaxis.set_major_locator(MultipleLocator(xran[2]))
    ax1.xaxis.set_minor_locator(MultipleLocator(xran[3]))
    ax1.set_xlim(xran[0],xran[1])
    ax1.set_ylim(yran[0],yran[1])

    if i%ncol==0         : ax1.set_ylabel(yl,fontsize=10)
    if i>ncol*(nrow-1)-1 : ax1.set_xlabel(xl,fontsize=10)
    ax1.grid(True)


##### titles for the figures
    if pkg_plot[5]==1:
      vmax,vmin = maxmin_ran([xran[0],xran[1]],[yran[0],yran[1]],xx,yy,np.log10(val[i]*pkg_plot[2]))
    else :
      vmax,vmin = maxmin_ran([xran[0],xran[1]],[yran[0],yran[1]],xx,yy,val[i]*pkg_plot[2])

    lmax = "Max: %.2f "%vmax
    lmin = "Min: %.2f "%vmin
    print (vstr[i],lmax,lmin)

#   if i==0:  ax1.set_title(title,fontsize=10)
#plt.figtext(0.5,0.95,title,fontsize=13,ha='center',va='bottom')
    if i==0: ax1.text(0.0,1.1,title,fontsize=10,ha='left', va='bottom',transform=ax1.transAxes)
    ax1.text(1.0,1.0,pkg_plot[0],fontsize=10,ha='right',va='bottom',transform=ax1.transAxes)
    ax1.text(0.0,0.0,lmin       ,fontsize=10,ha='left' ,va='bottom',transform=ax1.transAxes)
    ax1.text(1.0,0.0,lmax       ,fontsize=10,ha='right',va='bottom',transform=ax1.transAxes)


  plt.savefig(fps)
  buf=io.BytesIO()
  pl.dump(fig,buf)
#  buf.seek(0)

  return buf

def mhd_info(Dps,info,fmhd,pn,lp,xran,yran,tash,fMPBS):
  "set information for mhd plots            \
   read_2DMHD gives MHD values in the following orders        \
   vv   = [ bx,by,bz,bt,vx,vy,vz,vt,ex,ey,ez,et,jx,jy,jz,jt,rr,pp,pdy,kev ] "


  nrow=4
  ncol=1

  xx,yy,vv,vstr = read_2DMHD(fmhd,pn)
  mpbs=pd.read_csv(fMPBS)



  val1=[ [vv[3],vv[16],vv[19],vv[7]]]

  vstr1=[ [vstr[3],vstr[16],vstr[19],vstr[7]]]

  fname=[ 'btrtvt'   ]

# nrow = 1
# ncol = 1
# val1 = [[vv[16]]]
# vstr1= [[vstr[16]]]
# fname= ['rr']

  time = mhdt2tstr(locale.atoi(info.mhdt)+tash,info.ti)
  mhdt = mhdtime(time)

  if tash==0: title= mhdt.name+'\n'+info.run+'.'+pn+'_'+str(lp)+'.'+info.mhdt
  else: title= mhdt.name+': %dmin shifted\n'%int(tash/60)+info.run+'.'+pn+'_'+str(lp)+'.'+info.mhdt

  print ('============================================================================')
  for i in range(len(val1)) :

    Dps1=Dps+'/'+fname[i]
    if not os.path.exists(Dps1): os.system('mkdir '+Dps1)

    fps = Dps1+'/'+info.run+'.'+fname[i]+'.'+info.mhdt+'.'+pn+'_'+str(lp)+'.png'
    print ("SAVE ",fps)

    buf=mhd_2Dplot(nrow,ncol,fps,pn,lp,xran,yran,xx,yy,val1[i],vstr1[i],title,mpbs)

  return buf

def mhd_plot_main(argv):

  fname = argv[0].split(':')          # list of filenames
  fgrd  = fname[0]
  f3df  = fname[1]
  Dmhd  = fname[2]      # directory of openggcm ascii file
#  print(Dmhd)
  Dps   = fname[3]                      # directory to save plots


  vran  = argv[1].split(':')            # value range: pn:lp:xi:xf:yi:yf:ds
  pn    = vran[0]                       # plane : xy,xz,yz,xr
  lp    = locale.atof(vran[1])          # plane location
  xi    = locale.atof(vran[2])
  xf    = locale.atof(vran[3])
  yi    = locale.atof(vran[4])
  yf    = locale.atof(vran[5])
  ds    = locale.atof(vran[6])


  pran  = argv[2].split(':')          # plot range: x1:x2:dx1:dx2:y1:y2:dy1:dy2:re
  x1    = locale.atof(pran[0])    # xrange x1-x2
  x2    = locale.atof(pran[1])
  dx1   = locale.atof(pran[2])    # major x-tick interval
  dx2   = locale.atof(pran[3])    # minor x-tick interval
  y1    = locale.atof(pran[4])    # yrange y1-y2
  y2    = locale.atof(pran[5])
  dy1   = locale.atof(pran[6])    # major y-tick interval
  dy2   = locale.atof(pran[7])    # minor y-tick interval
  xran  = [x1,x2,dx1,dx2]
  yran  = [y1,y2,dy1,dy2]

  tash  = locale.atoi(argv[3])

  info  = runinfo(f3df)
  fmhd  = Dmhd+'/'+info.run+'.'+pn+'_'+vran[1]+'.'+info.mhdt
  fMPBS = Dmhd+'/modeling/'+info.run+'.MPBS'+pn+'_car.'+info.mhdt
  #print('fmhd',fmhd)
  Dps1  = Dps+'/'+pn+'_'+vran[1]
  if not os.path.exists(Dps1): os.system('mkdir '+Dps1)

  buf=mhd_info(Dps1,info,fmhd,pn,lp,xran,yran,tash,fMPBS)

  return buf

x=np.linspace(0,20)

for i in range(1,7):
    a=pd.read_csv('/Volumes/easystore/openggcm_run/Msh_Nstep/modeling/Msh_data_0.1.'+str(1200+1200*i).zfill(6),header=None,sep='\s+',names=['the','phi','m','rad','x','y','z','bx1','by1','bz1','vx1','vy1','vz1','rr1','pp1','ex1','ey1','ez1','xjx1','xjy1','xjz1','resis1']).drop_duplicates()
    a2=a[a.the==0]
    b=pd.read_csv('/Volumes/easystore/openggcm_run/Msh_Nstep/sheath_data_ascii/cn%d.txt' % i,sep='\s+',header=None,names=['theta_1','theta_2','f_1','f_2','B','ni','Ti','V'],skiprows=1)
    b['phi1']=np.where(b.theta_1<=180,b.theta_1,b.theta_1-360)
    b['phi2']=np.where(b.theta_2<=180,b.theta_2,b.theta_2-360)
    b['phi']=(b.phi1+b.phi2)/2
    b['f']=(b.f_1+b.f_2)/2
#    b2=b[(-90<=b.phi) & (b.phi<=90)]
    b2=b

    buf=mhd_plot_main([':/Msh_Nstep.3df.'+str(1200+1200*i).zfill(6)+':/Volumes/easystore/openggcm_run/Msh_Nstep:.',
        'xy:0:-40:20:-30:30:.2','-40:20:10:1:-30:30:10:1','0'])

    for phi in np.arange(-90+7.5,90,7.5):
        y=x*np.tan(phi*np.pi/180)

        b3=b2[(b2.phi<phi+.01) & (b2.phi>phi-.01)]
        b4=b3[['f','B','ni','Ti','V']].set_index('f')
        b4.columns=['B_T','ni_T','Ti_T','V_T']

        df_list=[]
        for j in range(1,10):
            a_t=a2[(a2.phi<phi+7.5) & (a2.phi>phi-7.5)]
            a_t2=a_t[(a_t.m>j-.1) & (a_t.m<j+.1)]
            a_t3=a_t2.mean().to_frame().T
            a_t3.phi=phi
            a_t3['f']=j/10
            df_list.append(a_t3)
        a3=pd.concat(df_list,axis=0).set_index('f')

        a4=a3.reindex(a3.index.union(b3.f))
        a5=a4.interpolate('index').reindex(index=b3.f)
        a5['B']=np.sqrt(a5.bx1**2+a5.by1**2+a5.bz1**2)
        a5['Ti']=72429.0*a5.pp1/a5.rr1/11600.
        a5['V']=np.sqrt(a5.vx1**2+a5.vy1**2+a5.vz1**2)
        a5['ni']=a5.rr1
        a6=a5[['B','ni','Ti','V']]

        comp=pd.concat([a6,b4],axis=1)
        comp2=comp[['B_T','B','ni_T','ni','Ti_T','Ti','V_T','V']]
        comp2.to_csv('/Volumes/easystore/openggcm_run/Msh_Nstep/modeling/comp_phi_0.1_%.1f_' % phi+str(1200+1200*i).zfill(6),float_format='%.3f')

        buf.seek(0)
        fig=pl.load(buf)
        fig.axes[0].plot(x,y,color='green')
        fig.axes[2].plot(x,y,color='green')
        fig.axes[4].plot(x,y,color='green')
        fig.axes[6].plot(x,y,color='green')

        ax1=fig.add_subplot(4,2,2)
        ax2=fig.add_subplot(4,2,4)
        ax3=fig.add_subplot(4,2,6)
        ax4=fig.add_subplot(4,2,8)

        ax1.plot(comp2.index,comp2.B_T,'o-',label='THEMIS')
        ax1.plot(comp2.index,comp2.B,'o-',label='OpenGGCM')
        ax2.plot(comp2.index,comp2.ni_T,'o-',label='THEMIS')
        ax2.plot(comp2.index,comp2.ni,'o-',label='OpenGGCM')
        ax3.plot(comp2.index,comp2.Ti_T,'o-',label='THEMIS')
        ax3.plot(comp2.index,comp2.Ti,'o-',label='OpenGGCM')
        ax4.plot(comp2.index,comp2.V_T,'o-',label='THEMIS')
        ax4.plot(comp2.index,comp2.V,'o-',label='OpenGGCM')
        ax4.set_xlabel('f')
        ax1.set_ylabel('B (nT)')
        ax2.set_ylabel('n (cm$^{-3}$)')
        ax3.set_ylabel('T (eV)')
        ax4.set_ylabel('V (km/s)')
        ax1.grid()
        ax2.grid()
        ax3.grid()
        ax4.grid()
        fig.suptitle('$\phi$=%.1f' % phi)
        handles, labels = ax1.get_legend_handles_labels()
        fig.legend(handles, labels, loc='upper right',bbox_to_anchor=(0.9,0.05))
        fig.savefig('/Volumes/easystore/openggcm_run/Msh_Nstep/modeling/plots/comp/comp_%d_phi_0.1_%.1f.png' % (20*i+20,phi))
        plt.close()



#!/opt/local/bin/python
#
# plot xray rate

import os
import sys
HOME=os.getenv("HOME")
sys.path.append(HOME+"/svn/hkc-project/trunk/lib/")
sys.path.append(HOME+"/svn/hkc-project/trunk/src/mhd-openggcm")

import math
import locale
import numpy              as np
import matplotlib.pyplot  as plt
from matplotlib.colors       import LogNorm
from matplotlib.ticker       import MultipleLocator, FuncFormatter
from mpl_toolkits.axes_grid1 import make_axes_locatable

from read_rates               import read_mpbs,read_xray,read_ena, read_xraycut, rate_vpkg
from openggcm_info           import mhdtime,mhdt2tstr

def maxmin_ran(xran,yran,xx,yy,val):
  " calculate max and min values between xran and yran \
    xx,yy should be increasing order, ext. [1,2,3] "

  nx   = len(xx)
  ny   = len(yy)
  vmax =-1.e9
  vmin = 1.e9
  if yran[1]>yran[0]: yran1=yran[0];yran2=yran[1]
  if yran[1]<yran[0]: yran1=yran[1];yran2=yran[0]
  if xran[1]>xran[0]: xran1=xran[0];xran2=xran[1]
  if xran[1]<xran[0]: xran1=xran[1];xran2=xran[0]

  for i in range(ny):
   if yy[i] >= yran1 and yy[i] <= yran2 :
     for j in range(nx):
      if xx[j] >= xran1 and xx[j] <=xran2 :
        if vmax <= val[i][j] : vmax = val[i][j]
        if vmin >= val[i][j] : vmin = val[i][j]
  return vmax,vmin


def fmt(x, pos):
    a, b = '{:.2e}'.format(x).split('e')
    b = int(b)
    return r'$10^{{{}}}$'.format(b)


def rate_plot(nrow,ncol,fps,xran,yran,xx,yy,val,vstr,title,bl=[],mp=[]):

  fig = plt.figure(figsize=(8,8))
  for i in range(len(val)):
    #print(i)
    if vstr[i]=='xray':
        if i==1:
            xx0=xx[i]
            xx[i]=180-np.array(yy[i])
            yy[i]=np.array(xx0)-180
            val[i]=np.transpose(val[i])
#    print(np.shape(xx[i]),np.shape(yy[i]),np.shape(val[i]))
    #print(xx[i])

    ax1      = fig.add_subplot(nrow,ncol,i+1,aspect='equal')
    ax1.tick_params(axis='both',which='major',labelsize=7)
    x,y      = np.meshgrid(xx[i],yy[i])
    pkg_plot = rate_vpkg(vstr[i])
#    valc=val[i][:-1,:-1]

    if pkg_plot[5]==1:
      #print(x,y)
#      print(val[i]*pkg_plot[2])
      cs    = ax1.contourf(x,y,val[i]*pkg_plot[2],levels=pkg_plot[3], cmap=pkg_plot[1],norm=LogNorm())
#     cs    = ax1.pcolor(x,y,val[i]*pkg_plot[2],cmap=pkg_plot[1],	\
#             norm=LogNorm(vmin=pkg_plot[3][0],vmax=pkg_plot[3][-1]))
    else :
#      print(x,y,val[i]*pkg_plot[2])
#      cs    = ax1.contourf(x,y,val[i]*pkg_plot[2],levels=pkg_plot[3], cmap=pkg_plot[1],extend='both')
#      cs    = ax1.pcolormesh(x,y,val[i]*pkg_plot[2], cmap=pkg_plot[1])
      cs    = ax1.contourf(x,y,val[i]*pkg_plot[2],levels=pkg_plot[3], cmap=pkg_plot[1],extend='both')
#      print(np.min(x),np.max(x),np.min(y),np.max(y))
#      print(x,y)
#     cs    = ax1.pcolor(x,y,val[i]*pkg_plot[2],vmin=pkg_plot[3][0],vmax=pkg_plot[3][-1], \
#             cmap=pkg_plot[1]) #,extend='both')

    if len(bl)!=0 :
#     for j in range(len(bl[0])/15): pl = ax1.plot(bl[0][j*15],bl[1][j*15],'w-',lw=0.1)
      for j in range(len(bl[0])/1): pl = ax1.plot(bl[0][j*1],bl[1][j*1],'k-',lw=0.1)
    if len(mp)!=0 :
      pl1 = ax1.plot(mp[0],mp[1],'k-')
      pl2 = ax1.plot(mp[2],mp[3],'k-')

    cax1  = make_axes_locatable(ax1)
    cax2  = cax1.append_axes("right",size="3%",pad="5%")
    if pkg_plot[5]==1:
        cb    = fig.colorbar(cs,cax=cax2,ticks=pkg_plot[4],format=FuncFormatter(fmt))
    else:
        cb    = fig.colorbar(cs,cax=cax2,ticks=pkg_plot[4])
    cl    = plt.getp(cb.ax, 'ymajorticklabels')
    plt.setp(cl, fontsize=7)



    ax1.xaxis.set_major_locator(MultipleLocator(xran[2]))
    ax1.xaxis.set_minor_locator(MultipleLocator(xran[3]))
    ax1.yaxis.set_major_locator(MultipleLocator(yran[2]))
    ax1.yaxis.set_minor_locator(MultipleLocator(yran[3]))
    ax1.set_xlim(xran[0],xran[1])
    ax1.set_ylim(yran[0],yran[1])
#    print(xran,yran)
    if vstr[i]!='xraycut':
        if i==1:
            ax1.set_xlabel('Polar angle [deg]',fontsize=10)
            ax1.set_ylabel('Azimuth angle [deg]', fontsize=10)
        else:
            ax1.set_ylabel('Polar angle [deg]',fontsize=10)
            ax1.set_xlabel('Azimuth angle [deg]', fontsize=10)
    else:
        ax1.set_ylabel('Ygse [Re]',fontsize=10)
        ax1.set_xlabel('Xgse [Re]', fontsize=10)
    ax1.grid(True)

##### titles for the figures
    vmax,vmin = maxmin_ran([xran[0],xran[1]],[yran[0],yran[1]],xx[i],yy[i],val[i])

    if vstr[i]!='xraycut':
        lmax = "Max: %.3f "%(vmax*pkg_plot[2])
        lmin = "Min: %.3f "%(vmin*pkg_plot[2])
    else:
        lmax = "Max: %.3e "%(vmax*pkg_plot[2])
        lmin = "Min: %.3e "%(vmin*pkg_plot[2])

    print(vstr[i],lmax,lmin)

#   if i==0: plt.figtext(0.5,0.95,title,fontsize=13,ha='center',va='bottom')
#ax1.set_title(title,fontsize=10)
    ax1.text(0.0,1.0,title[i],      fontsize=10,ha='left', va='bottom',transform=ax1.transAxes)
    ax1.text(1.0,1.0,pkg_plot[0],fontsize=10,ha='right',va='bottom',transform=ax1.transAxes)
    if vstr[i]!='xraycut':
        ax1.text(0.0,0.0,lmin       ,fontsize=10,ha='left' ,va='bottom',transform=ax1.transAxes,color='white')
        ax1.text(1.0,0.0,lmax       ,fontsize=10,ha='right',va='bottom',transform=ax1.transAxes,color='white')
    else:
        ax1.text(0.0,0.0,lmin       ,fontsize=10,ha='left' ,va='bottom',transform=ax1.transAxes,color='black')
        ax1.text(1.0,0.05,lmax       ,fontsize=10,ha='right',va='bottom',transform=ax1.transAxes,color='black')


#    ax1.invert_xaxis()
 # ax1.ticklabel_format(useOffset=False)
  plt.tight_layout()
  print("Save ",fps)
  plt.savefig(fps)
  plt.close()

  return

def rate_info(fps,fin,fbl,fmp,vstr,xran,yran,tash):
  "set information for mhd plots                                                "

  nrow  = 1
  ncol  = len(vstr)
#  print(fin[0])
#  info  = runinfo(fin[0])
#  time  = mhdt2tstr(locale.atoi(info.mhdt)+tash,info.ti)
 # mhdt  = mhdtime(time)

# fps = Dps+'/'+fin.split('/')[-1]+'.ps'
# print "plot ",fps
  phi={}
  lat={}
  rate1={}
  sc={}
  rate={}
  stitle={}
  rate2={}
  title={}
  fname={}
  for i in range(len(vstr)):
    if   vstr[i]=='ena' :
        phi[i],lat[i],rate1[i],sc[i] = read_ena(fin[i])
        rate[i]             = rate1[i][0]
        stitle[i]           = vstr[i]+'%5.2fkeV_x%02dy%02dz%02d'%(sc[i][-1]/1e3,sc[i][0],sc[i][1],sc[i][2])
    elif vstr[i]=='xray':
        phi[i],lat[i],rate[i],sc[i] = read_xray(fin[i])
        if i==0:
            stitle[i]          = vstr[i]+'_side%02d%02d%02d'%(sc[i][0],sc[i][1],sc[i][2])
        else:
            stitle[i]          = vstr[i]+'_polar%02d%02d%02d'%(sc[i][0],sc[i][1],sc[i][2])
    elif vstr[i]=='xraycut':
        phi[i],lat[i],rate[i]    = read_xraycut(fin[i])
        stitle[i]          = vstr[i]+'_cut'
  if vstr[0]=='xraycut':
    prop=np.empty_like(rate[0])
    prop2=np.empty_like(rate[0])
    count_0,count,count_nan=0,0,0
    for index,value in np.ndenumerate(rate[0]):
        prop[index]=abs(value-rate[1][index])/rate[1][index]
        count_0+=1
        if value>0:
            prop2[index]=abs(value-rate[1][index])/rate[1][index]
            count+=1
        elif np.isnan(value):
            count_nan+=1
        #else:print(prop[index])
    print('prop',np.nanmean(prop),np.nanmean(prop2))
#    print(prop,prop2)
    print('count',count_0,count,count_nan)
    print("test max,min", np.nanmax(rate[i]),np.nanmin(rate[i]))



  for i in range(len(vstr)):
      title[i] = stitle[i]
      if vstr[i]=='xraycut':
        if i==0: title[i]='Mshpy\n'
        elif i==1: title[i]='Jorgensen2019\n'
  #    fname = fps+'/'+info.run+'.'+stitle+'.'+info.mhdt+'.ps'
  fname = fps+'/'+fin[0].split('/')[-1]+'_2.png'
  print(fname)

  bl=[];mp=[]
  if fbl!='none': bl  = read_bline(fbl,sc)
  if fmp!='none': mp  = read_mpbs(fmp,sc)


# print len(phi),len(lat),len(rate),len(rate[0])
  rate_plot(nrow,ncol,fname,xran,yran,phi,lat,rate,vstr,title,bl=bl,mp=mp)

  return


def main(argv):

  fps   = argv[0]                       # plot directory
  fin   = argv[1].split(':')		# input file
  print(fin)
  fbl   = argv[2]			# input file for bl
  fmp   = argv[3]			# input file for BS,MP
  pran  = argv[4].split(':')            # plot range: type:x1:x2:dx1:dx2:y1:y2:dy1:dy2
  tash  = locale.atoi(argv[5])

  vstr  = pran[0].split(',') 			# xray or ena
  x1    = locale.atof(pran[1])          # xrange x1-x2
  x2    = locale.atof(pran[2])
  dx1   = locale.atof(pran[3])          # major x-tick interval
  dx2   = locale.atof(pran[4])          # minor x-tick interval
  y1    = locale.atof(pran[5])          # yrange y1-y2
  y2    = locale.atof(pran[6])
  dy1   = locale.atof(pran[7])          # major y-tick interval
  dy2   = locale.atof(pran[8])          # minor y-tick interval
  xran  = [x1,x2,dx1,dx2]
  yran  = [y1,y2,dy1,dy2]

  if vstr=='dena' or vstr=='dxray':
     fin.append(fin[0][:-6]+'%06d'%locale.atoi(pran[9]));print(fin)
  rate_info(fps,fin,fbl,fmp,vstr,xran,yran,tash)

  return


if __name__ == "__main__":
  main(sys.argv[1:])



#!/usr/bin/python
# read WIND MFI and SWE data
# read ACe MFI and SWE data

import os
import sys
HOME=os.getenv("HOME")
sys.path.append(HOME+"/svn/hkc-project/trunk/lib/")


import locale
import matplotlib.pyplot as plt
from jrmod01  		import *
from numpy		import arange
from openggcm_info 	import mhdtime,mhdt2hrmi

def ldaf(filename):
    with open(filename) as myFile:
        for num, line in enumerate(myFile, 1):
            if line.startswith('dd'):
                lda=num
#                print ('found at line:', num)
    return lda

def read_GE(fi,lda,setbx=9999.99):
  "read Geotail data 								"

  BXerr=9999.99
  BErr='9999.99'                        # err of B field
  VErr='99999.9'                        # err of Velocity
  NErr='999.99'                         # err of Number density
  TErr='9999999.'                       # err of Temparature
  LErr='9999.99'                        # err of Satellite Location
  re  = 6.37814e3

  f=open(fi,'r')
  l=f.read().split('\n')
  f.close()

  print("===============  CLEAN ALL UNNECESSARY INFORMATION  ===============")
  print("REMOVE ALL HEADERS BEFORE")
  print(l[lda])
  del   l[:lda]
  print("REMOVE ALL THE FOLLOWING TAILS ")
  print(l[-4:])
  del   l[-4:]
  print("-------------------------------------------------------------------")
  print("THE FIRST & LAST DATA LINES ARE")
  print(l[0])
  print(l[-1])
  print("===================================================================")



  date=[]
  time=[]
  bx=[];by=[];bz=[]
  vx=[];vy=[];vz=[]
  xx=[];yy=[];zz=[]
  nn=[];pp=[];tk=[]

  for s in l:
    w=s.split(None)

    if w[2]!=BErr  and w[3]!=BErr  and w[4]!=BErr  and \
       w[5]!=VErr  and w[6]!=VErr  and w[7]!=VErr  and \
       w[8]!=NErr  and w[9]!=TErr  and			\
       w[10]!=LErr and w[11]!=LErr and w[12]!=LErr :

       date.append(w[0])
       time.append(w[1])

       print(setbx)
       if setbx==BXerr  : bx.append(w[2])
       else             : bx.append(str(setbx))
       by.append(w[3])
       bz.append(w[4])

       vx.append(w[5])
       vy.append(w[6])
       vz.append(w[7])

       nn.append(w[8])
       tk.append(w[9])

       print(w[8])
       nn1=locale.atof(w[8])
       tk1=locale.atof(w[9])
       pp1=nn1*1e6*tk1*1.380654e-23*1e12
       pp.append(locale.format('%f',pp1))

       xx.append(w[10])
       yy.append(w[11])
       zz.append(w[12])

  return  date,time,bx,by,bz,vx,vy,vz,nn,pp,tk,xx,yy,zz


def read_ACMFI(fi,lda,setbx=9999.99) :
  "read ACE MFI data                                                            \
   fi     : ACE/MFI data in the following order                                 \
            [ date,time,bx,by,bz,xx,yy,zz ]                                     \
   lda    : line number where data started                                      \
   setbx  : input setbx into IMF bx if setbx!=99999                             "


  BXerr=9999.99
  Err = '-1.00000E+31'
  re  = 6.37814e3

  f=open(fi,'r')
  l=f.read().split('\n')
  f.close()

  print("===============  CLEAN ALL UNNECESSARY INFORMATION  ===============")
  print("REMOVE ALL HEADERS BEFORE")
  print(l[lda])
  del   l[:lda]
  print("REMOVE ALL THE FOLLOWING TAILS ")
  print(l[-4:])
  del   l[-4:]
  print("-------------------------------------------------------------------")
  print("THE FIRST & LAST DATA LINES ARE")
  print(l[0])
  print(l[-1])
  print("===================================================================")


  date=[]
  time=[]
  bx=[];by=[];bz=[]
  xl=[];yl=[];zl=[]

  for s in l:
    w=s.split(None)
    if len(w)>2  and w[2]!=Err and w[3]!=Err and w[4]!=Err and \
       w[5]!=Err and w[6]!=Err and w[7]!=Err                   :

      date.append(w[0])
      time.append(w[1])
      if setbx==BXerr:  bx.append(w[2])
      else           :  bx.append(str(setbx))
      by.append(w[3])
      bz.append(w[4])

      x1=locale.format('%f',locale.atof(w[5])/re)
      y1=locale.format('%f',locale.atof(w[6])/re)
      z1=locale.format('%f',locale.atof(w[7])/re)
      xl.append(w[5])
      yl.append(w[6])
      zl.append(w[7])

  return date,time,bx,by,bz,xl,yl,zl


def read_ACSWE(fi,lda) :
  " read ACE SWE data only if all plasma data are not Err values             	\
    fi  : ACE/SWE data in the following order                                  	\
          [ date,time,np,Tk,vxgse,vygse,vzgse ]                 		\
    lda : the line number where data starts                                     "

  Err = '-1.00000E+31'

  f=open(fi,'r')
  l=f.read().split('\n')
  f.close()

  print("===============  CLEAN ALL UNNECESSARY INFORMATION  ===============")
  print("REMOVE ALL HEADERS BEFORE")
  print(l[lda])
  del   l[:lda]
  print("REMOVE ALL THE FOLLOWING TAILS ")
  print(l[-4:])
  del   l[-4:]
  print("-------------------------------------------------------------------")
  print("THE FIRST & LAST DATA LINES ARE")
  print(l[0])
  print(l[-1])
  print("===================================================================")

  date=[]
  time=[]
  np=[];pp=[];tk=[]
  vx=[];vy=[];vz=[];

  for s in l:
    w=s.split(None)
    if len(w)>2  and w[2]!=Err and w[3]!=Err and  \
       w[4]!=Err and w[5]!=Err and w[6]!=Err      :

      date.append(w[0])
      time.append(w[1])
      np.append(w[2])
      vx.append(w[4])
      vy.append(w[5])
      vz.append(w[6])

      np1=locale.atof(w[2])
      tk1=locale.atof(w[3])
      tk2=tk1*1.380654e-23/1.602176e-19
      pp1=np1*1e6*tk1*1.380654e-23*1e12
      pp.append(locale.format('%f',pp1))
      tk.append(locale.format('%f',tk2))

  return date,time,np,pp,tk,vx,vy,vz


def read_allACSWE(fi,lda) :
  " read all available non-Err ACE SWE data                                     \
    fi  : ACE/SWE data in the following order                                   \
          [ date,time,np,Tk,vxgse,vygse,vzgse ]                                 \
    lda : the line number where data starts                                     "

  ERR = '-1.00000E+31'

  f=open(fi,'r')
  l=f.read().split('\n')
  f.close()

  print("===============  CLEAN ALL UNNECESSARY INFORMATION  ===============")
  print("REMOVE ALL HEADERS BEFORE")
  print(l[lda])
  del   l[:lda]
  print("REMOVE ALL THE FOLLOWING TAILS ")
  print(l[-4:])
  del   l[-4:]
  print("-------------------------------------------------------------------")
  print("THE FIRST & LAST DATA LINES ARE")
  print(l[0])
  print(l[-1])
  print("===================================================================")

  ddnp=[];ddpp=[];ddtk=[];ddvx=[];ddvy=[];ddvz=[]
  ttnp=[];ttpp=[];tttk=[];ttvx=[];ttvy=[];ttvz=[]
  np=[];pp=[];tk=[]
  vx=[];vy=[];vz=[];

  for s in l:
    w=s.split(None)
    if len(w)>2 :
      if w[2]!=ERR: ddnp.append(w[0]);ttnp.append(w[1]);np.append(w[2])
      if w[4]!=ERR: ddvx.append(w[0]);ttvx.append(w[1]);vx.append(w[4])
      if w[5]!=ERR: ddvy.append(w[0]);ttvy.append(w[1]);vy.append(w[5])
      if w[6]!=ERR: ddvz.append(w[0]);ttvz.append(w[1]);vz.append(w[6])
      if w[3]!=ERR:
        tk1=locale.atof(w[3])
        tk2=tk1*1.380654e-23/1.602176e-19
        ddtk.append(w[0]);tttk.append(w[1])
        tk.append(locale.format('%f',tk2))
        if w[2]!=ERR:
          np1=locale.atof(w[2])
          pp1=np1*1e6*tk1*1.380654e-23*1e12
          ddpp.append(w[0]);ttpp.append(w[1])
          pp.append(locale.format('%f',pp1))
  date=[ddnp,ddpp,ddtk,ddvx,ddvy,ddvz]
  time=[ttnp,ttpp,tttk,ttvx,ttvy,ttvz]
  return date,time,np,pp,tk,vx,vy,vz

def read_WIMFI(fi,lda,setbx=9999.99) :
  "read WIND MFI data								\
   fi     : WIND/MFI data in the following order 				\
            [ date,time,bx,by,bz ]						\
   lda    : line number where data started					\
   setbx  : input setbx into IMF bx if setbx!=99999				"

  BXerr=9999.99
  Err='-1.00000E+31'

  f=open(fi,'r')
  l=f.read().split('\n')
  f.close()

  print("===============  CLEAN ALL UNNECESSARY INFORMATION  ===============")
  print("REMOVE ALL HEADERS BEFORE")
  print(l[lda])
  del   l[:lda]
  print("REMOVE ALL THE FOLLOWING TAILS ")
  print(l[-4:])
  del   l[-4:]
  print("-------------------------------------------------------------------")
  print("THE FIRST & LAST DATA LINES ARE")
  print(l[0])
  print(l[-1])
  print("===================================================================")

  date=[]
  time=[]
  bx=[];by=[];bz=[]

  for s in l:
    w=s.split(None)
    if len(w)>=2 and w[2]!=Err and w[3]!=Err and w[4]!=Err:
      date.append(w[0])
      time.append(w[1])
      if setbx==BXerr:  bx.append(w[2])
      else           :  bx.append(str(setbx))
      by.append(w[3])
      bz.append(w[4])

  return date,time,bx,by,bz


def read_WISWE(fi,lda):
  " read WIND SWE data 								\
    fi  : WIND/SWE data in the following order					\
          [ date,time,vth,np,xgse,ygse,zgse,vxgse,vygse,vzgse ]			\
    lda : the line number where data starts					\
    ----------------------------------						\
    CHANGES: As of Jul 14 2016, SWE data format changes, including DEL_TIME.	\
             [date,time,vth,np,DEL_TIME,xgse,ygse,zgse,vxgse,vygse,vzgse]    	\
             For swdata calculation, DEL_TIME is ignored 			"


  Err='-1.00000E+31'
  re=6.37814e3

  f=open(fi,'r')
  l=f.read().split('\n')
  f.close()

  print("===============  CLEAN ALL UNNECESSARY INFORMATION  ===============")
  print("REMOVE ALL HEADERS BEFORE")
  print(l[lda])
  del   l[:lda]
  print("REMOVE ALL THE FOLLOWING TAILS ")
  print(l[-4:])
  del   l[-4:]
  print("-------------------------------------------------------------------")
  print("THE FIRST & LAST DATA LINES ARE")
  print(l[0])
  print(l[-1])
  print("===================================================================")


  date=[]
  time=[]
  th=[];np=[];pp=[]
  xl=[];yl=[];zl=[]
  vx=[];vy=[];vz=[]

  print(len(l))
  for s in l:
    w=s.split(None)
    if len(w)>2 and 						\
       w[2]!=Err and w[3]!=Err and w[4]!=Err and w[5]!=Err and 	\
       w[6]!=Err and w[7]!=Err and w[8]!=Err and w[9]!=Err      :

      date.append(w[0])
      time.append(w[1])

      th.append(w[2])	# thermal velocity
      np.append(w[3])

      x1=locale.atof(w[5])/re
      y1=locale.atof(w[6])/re
      z1=locale.atof(w[7])/re
      xl.append(locale.format('%f',x1))
      yl.append(locale.format('%f',y1))
      zl.append(locale.format('%f',z1))

      vx.append(w[8])
      vy.append(w[9])
      vz.append(w[10])

# calculate thermal pressure
# Note that OpenGGCM needs THERMAL pressure, not DYNAMIC pressure

      th1  = locale.atof(w[2])
      np1  = locale.atof(w[3])
      pp1  = np1*1e6*(th1*1e3)**2*1.672614e-27/2*1e12

      pp.append(locale.format('%f',pp1))

  return date,time,th,np,xl,yl,zl,vx,vy,vz,pp


def read_THFGM(fi, lda,setbx=9999.99):
  "read THEMIS FGM data                                                           \
   fi     : THEMIS FGM data in the following order                                \
            [ date,time,Btot_goodquality,Btot_allquality,UT_seconds,bxgse_good,bygse_good,bzgse_good ]     \
   lda    : line number where data started                                      \
   setbx  : input setbx into IMF bx if setbx!=99999                             "

  BXerr=9999.99
  Err='-1.00000E+31'

  f=open(fi,'r')
  l=f.read().split('\n')
  f.close()

  print("===============  CLEAN ALL UNNECESSARY INFORMATION  ===============")
  print("REMOVE ALL HEADERS BEFORE")
  print(l[lda])
  del   l[:lda]
  print("REMOVE ALL THE FOLLOWING TAILS ")
  print(l[-4:])
  del   l[-4:]
  print("-------------------------------------------------------------------")
  print("THE FIRST & LAST DATA LINES ARE")
  print(l[0])
  print(l[-1])
  print("===================================================================")

  date=[]
  time=[]
  bx=[];by=[];bz=[]

  for s in l:
    w=s.split(None)
    if len(w)>=2 and w[2]!=Err and w[3]!=Err and w[4]!=Err:
      date.append(w[0])
      time.append(w[1])
      if setbx==BXerr:  bx.append(w[5])
      else           :  bx.append(str(setbx))
      by.append(w[6])
      bz.append(w[7])

  return date,time,bx,by,bz


def read_THESA(fi,lda):
  " read THEMIS ESA data                                                          \
    fi  : WIND/SWE data in the following order                                  \
          [ date,time,vth,np,xgse,ygse,zgse,vxgse,vygse,vzgse ]                 \
    lda : the line number where data starts                                     \
    ----------------------------------                                          \
    CHANGES: As of Jul 14 2016, SWE data format changes, including DEL_TIME.    \
             [date,time,vth,np,DEL_TIME,xgse,ygse,zgse,vxgse,vygse,vzgse]       \
             For swdata calculation, DEL_TIME is ignored                        "


  Err='-1.00000E+31'
  re=6.37814e3

  f=open(fi,'r')
  l=f.read().split('\n')
  f.close()

  print("===============  CLEAN ALL UNNECESSARY INFORMATION  ===============")
  print("REMOVE ALL HEADERS BEFORE")
  print(l[lda])
  del   l[:lda]
  print("REMOVE ALL THE FOLLOWING TAILS ")
  print(l[-4:])
  del   l[-4:]
  print("-------------------------------------------------------------------")
  print("THE FIRST & LAST DATA LINES ARE")
  print(l[0])
  print(l[-1])
  print("===================================================================")


  date=[]
  time=[]
  np=[];pp=[];tk=[];teV=[]
  vx=[];vy=[];vz=[]

  print(len(l))
  for s in l:
    w=s.split(None)
    if len(w)>2 and                                             \
       w[2]!=Err and w[3]!=Err and w[4]!=Err and w[5]!=Err and  \
       w[6]!=Err and w[7]!=Err and w[8]!=Err and w[9]!=Err      :

      date.append(w[0])
      time.append(w[1])

      np.append(w[2])
      teV.append(w[3])
      vx.append(w[6])
      vy.append(w[7])
      vz.append(w[8])

# calculate thermal pressure
# Note that OpenGGCM needs THERMAL pressure, not DYNAMIC pressure

      np1  = locale.atof(w[2])
      tk1  = locale.atof(w[3])*11600.
      pp1  = tk1*np1/72429.
      tk.append(locale.format('%f',tk1))
      pp.append(locale.format('%f',pp1))

  return date,time,np,pp,tk,vx,vy,vz


  return

def read_omni(fi,lda,setbx=9999.99):

  BXerr=9999.99
  BErr='9999.99'			# err of B field
  VErr='99999.9'			# err of Velocity
  NErr='999.99'				# err of Number density
  TErr='9999999.'			# err of Temparature
  LErr='9999.99'			# err of Satellite Location

  print("READ ",fi)
  f=open(fi,'r')
  l=f.read().split('\n')
  f.close()

  print("===============  CLEAN ALL UNNECESSARY INFORMATION  ===============")
  print("REMOVE ALL HEADERS BEFORE")
  print(l[lda])
  del   l[:lda]
#  print("REMOVE ALL THE FOLLOWING TAILS ")
#  print(l[-4:])
#  del   l[-4:]
  print("-------------------------------------------------------------------")
  print("THE FIRST & LAST DATA LINES ARE")
  print(l[0])
  print(l[-1])
  print("===================================================================")


  dateb=[]
  timeb=[]
  datev=[]
  timev=[]
  datel=[]
  timel=[]
  bx=[];by=[];bz=[]
  vx=[];vy=[];vz=[]
  xx=[];yy=[];zz=[]
  nn=[];tk=[];pp=[]


  for s in l:

    w=s.split(None)
    if len(w)>2 :

     yr1=locale.atoi(w[0])
     dy1=locale.atoi(w[1])
     hh1=locale.atoi(w[2])
     mi1=locale.atoi(w[3])

     mm1,dd1=dayofyear2date(yr1,dy1)

     date1='%02d-%02d-%04d'%(dd1,mm1,yr1)
     time1='%02d:%02d:0.0'%(hh1,mi1)

     print(date1,time1)

     if w[4]!=BErr and w[5]!=BErr and w[6]!=BErr:
       dateb.append(date1)
       timeb.append(time1)
       if setbx==BXerr: bx.append(w[4])
       else           : bx.append(str(setbx))
       by.append(w[5])
       bz.append(w[6])


     if  w[7] !=VErr and w[8] !=VErr and w[9] !=VErr and  \
         w[10]!=NErr and w[11]!=TErr                      :

       datev.append(date1)
       timev.append(time1)

       vx.append(w[7])
       vy.append(w[8])
       vz.append(w[9])

       nn.append(w[10])
       tk.append(w[11])

       nn1=locale.atof(w[10])
       tk1=locale.atof(w[11])
       pp1=nn1*1e6*tk1*1.380654e-23*1e12
       pp.append(locale.format('%f',pp1))


     if  w[12]!=LErr and w[13]!=LErr and w[14]!=LErr :
       datel.append(date1)
       timel.append(time1)
       xx.append(w[12])
       yy.append(w[13])
       zz.append(w[14])

  return  dateb,timeb,bx,by,bz,datev,timev,vx,vy,vz,nn,pp,tk,datel,timel,xx,yy,zz



def dayofyear2date(yy,dy):
  "convert day of year to date: 2005:135 -> 2005:05:15"

  if yy%4==0 : day=[31,29,31,30,31,30,31,31,30,31,30,31]
  else       : day=[31,28,31,30,31,30,31,31,30,31,30,31]


  for i in range(len(day)):
    if dy < day[0] :
      mm=1
      break

    dy=dy-day[i]

    if dy < day[i+1] :
      mm=i+2
      break

  return mm,dy


def main():

  yy=2005
  dy=135

  mm,dd=dayofyear2date(2005,135)

  print("year, dayofyear :",yy,dy)
  print("year, month, day:",yy,mm,dd)

  return


if __name__=='__main__':

  main()



import numpy as np
import matplotlib.pyplot as plt
import pandas as pd

fig, axes = plt.subplots(2,4,subplot_kw={'projection': 'polar'},figsize=(10,6))
axes[1,3].axis("off")

for i in range(1,8):
    a=pd.read_csv('modeling/xz_cusp_cn_n.'+str(1200*i).zfill(6),header=None,sep='\s+',skiprows=1)
    b=pd.read_csv('modeling/xz_cusp_cn_s.'+str(1200*i).zfill(6),header=None,sep='\s+',skiprows=1)
    ax=axes.flat[i-1]
    ax.plot(a[2]*np.pi/180,a[0])
    ax.plot(a[3]*np.pi/180,a[0])
    ax.plot(b[2]*np.pi/180,b[0])
    ax.plot(b[3]*np.pi/180,b[0])
    ax.set_thetamin(-90)
    ax.set_thetamax(90)
    mask=plt.Circle((0,0),3.5,transform=ax.transData._b)
    ax.add_artist(mask)
    ax.set_ylim(0,12)
    if i==1:
        ax.title.set_text('n_sw=1cm$^{-3}$')
    else:
        ax.title.set_text('n_sw=%d'%(5*(i-1))+'cm$^{-3}$')


plt.savefig('Cusp_plot.png')



import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
import glob
import datetime
from scipy.spatial.transform import Rotation as R

# Bow Shock
a11=.45
a22=1
a33=.8
a12=.18
a14=46.6
a24=-2.2
a34=-.6
a44=-618
R0=11.8954

R_0=15.02
lam=1.17
epsilon=6.55

def appendSpherical_np(xyz):
    ptsnew = np.hstack((xyz,np.empty(xyz.shape)))
    xy = xyz[:,0]**2 + xyz[:,1]**2
    ptsnew[:,3] = np.sqrt(xy + xyz[:,2]**2)
    #ptsnew[:,4] = np.arctan2(np.sqrt(xy), xyz[:,2]) # for elevation angle defined from Z-axis down
    ptsnew[:,4] = np.arctan2(xyz[:,2], np.sqrt(xy)) # for elevation angle defined from XY-plane up
    ptsnew[:,5] = np.arctan2(xyz[:,1], xyz[:,0])
    return ptsnew

def mpdf(Pd,Bz,xyz):
    yz=xyz[:,1]**2+xyz[:,2]**2
    theta=np.arctan2(np.sqrt(yz),xyz[:,0])
    r0=(10.22+1.29*np.tanh(0.184*(Bz+8.14)))*Pd**(-1/6.6)
    alpha=(0.58-0.007*Bz)*(1+0.024*np.log(Pd))
    R = r0*(2/(1+np.cos(theta)))**alpha
    return R

def BSf_Farris(Pd,Bz,Mm,xyz):
    yz=xyz[:,1]**2+xyz[:,2]**2
    theta=np.arctan2(np.sqrt(yz),xyz[:,0])
    R_mp=(10.22+1.29*np.tanh(0.184*(Bz+8.14)))*Pd**(-1/6.6)
    R_bs=(1+1.1/4*(Mm**2+3)/Mm**2)*R_mp
    b=0.0223*(Pd/1.8)**(1/6)
    r_bs=(-np.cos(theta)+np.sqrt(np.cos(theta)**2+4*R_bs*b*np.sin(theta)**2))/(2*b*np.sin(theta)**2)
    return r_bs

def BSf_Farris_Jerab(Pd,N,V,B,Ma,xyz):
    yz=xyz[:,1]**2+xyz[:,2]**2
    theta=np.arctan2(np.sqrt(yz),xyz[:,0])
    D=0.937*(0.846+0.042*B)
    R_bs=91.55/(N*V**2)**(1/6)*(1+D*((5/3-1)*Ma**2+2)/((5/3+1)*(Ma**2-1)))

    b=0.0223*(Pd/1.8)**(1/6)
    r_bs=(-np.cos(theta)+np.sqrt(np.cos(theta)**2+4*R_bs*b*np.sin(theta)**2))/(2*b*np.sin(theta)**2)
    return r_bs

def bsdf_Jelinek(Pd,xyz):

    a_14=(4*R_0/lam**2*Pd**(-1/epsilon))
    a_44=(-1/4*lam**2*a_14**2)
    yz=xyz[:,1]**2+xyz[:,2]**2
    theta=np.arctan2(np.sqrt(yz),xyz[:,0])
    cos=np.cos(theta)
    sin=np.sin(theta)
    r=np.array((-a_14*cos+np.sqrt(a_14**2*cos**2-4*a_44*sin**2))/(2*sin**2))
    r0=-a_44/a_14
    r[np.isnan(r)]=r0[np.isnan(r)]
    return r

def KF94R(Pd,Bz,N,V,B,Ma,xyz,f_msh):
    yz=xyz[:,1]**2+xyz[:,2]**2
    theta=np.arctan2(np.sqrt(yz),xyz[:,0])
    phi=np.arctan2(xyz[:,2],xyz[:,1])

    R_mp=(10.22+1.29*np.tanh(0.184*(Bz+8.14)))*Pd**(-1/6.6)
    D=0.937*(0.846+0.042*B)
    R_bs=91.55/(N*V**2)**(1/6)*(1+D*((5/3-1)*Ma**2+2)/((5/3+1)*(Ma**2-1)))
    b=0.0223*(Pd/1.8)**(1/6)
    b_mp=b/(1+4*b*(R_mp-R_bs))

    r_mp=(-np.cos(theta)+np.sqrt(np.cos(theta)**2+4*R_mp*b_mp*np.sin(theta)**2))/(2*b_mp*np.sin(theta)**2)
    r_bs=(-np.cos(theta)+np.sqrt(np.cos(theta)**2+4*R_bs*b*np.sin(theta)**2))/(2*b*np.sin(theta)**2)
    print('R_mp & R_bs at the first data point:',r_mp.iloc[0],r_bs.iloc[0])

    r_KF=r_mp+f_msh*(r_bs-r_mp)
    #print(r_KF)
    xyz_KF=np.array([r_KF*np.cos(theta),r_KF*np.sin(theta)*np.cos(phi),r_KF*np.sin(theta)*np.sin(phi)]).T
    return xyz_KF

def KF94R_R(Pd,Bz,N,V,B,Ma,xyz_kf):
    #print(xyz_kf)
    yz=xyz_kf[:,1]**2+xyz_kf[:,2]**2
    theta=np.arctan2(np.sqrt(yz),xyz_kf[:,0])
    phi=np.arctan2(xyz_kf[:,2],xyz_kf[:,1])
    r_kf=np.sqrt(yz+xyz_kf[:,0]**2)
    R_mp=(10.22+1.29*np.tanh(0.184*(Bz+8.14)))*Pd**(-1/6.6)
    alpha=(0.58-0.007*Bz)*(1+0.024*np.log(Pd))
    #print(xyz_kf,R_mp,theta,alpha)
    r_mp=R_mp*(2/(1+np.cos(theta)))**alpha
    D=0.937*(0.846+0.042*B)
    R_bs=91.55/(N*V**2)**(1/6)*(1+D*((5/3-1)*Ma**2+2)/((5/3+1)*(Ma**2-1)))
    b=0.0223*(Pd/1.8)**(1/6)
    b_mp=b/(1+4*b*(R_mp-R_bs))


    r_mp_kf=(-np.cos(theta)+np.sqrt(np.cos(theta)**2+4*R_mp*b_mp*np.sin(theta)**2))/(2*b_mp*np.sin(theta)**2)
    r_bs=(-np.cos(theta)+np.sqrt(np.cos(theta)**2+4*R_bs*b*np.sin(theta)**2))/(2*b*np.sin(theta)**2)

#    print(r_kf,r_mp_kf,r_bs)
    f_kf=(r_kf-r_mp_kf)/(r_bs-r_mp_kf)
    #print(f_kf)

    r=r_mp+f_kf*(r_bs-r_mp)
    xyz=np.array([r*np.cos(theta),r*np.sin(theta)*np.cos(phi),r*np.sin(theta)*np.sin(phi)]).T
#    print('KF and GSE',xyz_kf,xyz)
    return xyz



def normalize(v):
    norm = np.linalg.norm(v)
    if norm == 0:
       return v
    return v / norm

def backtrack(Pd,Bz,N,B,Ma,v_x,v_y,v_z,xyz,f_msh):
    V=np.sqrt(v_x**2+v_y**2+v_z**2)
    R_mp=(10.22+1.29*np.tanh(0.184*(Bz+8.14)))*Pd**(-1/6.6)
    D=0.937*(0.846+0.042*B)
    R_bs=91.55/(N*V**2)**(1/6)*(1+D*((5/3-1)*Ma**2+2)/((5/3+1)*(Ma**2-1)))
    bl=0.0223*(Pd/1.8)**(1/6)
    df=2*R_bs-R_mp-1/(2*bl)
    R_bsl=R_bs-df
    R_mpl=R_mp-df
    xfl=R_mpl/2+df
    #R_bsl=R_bs
    #R_mpl=R_mp


    yz=xyz[:,1]**2+xyz[:,2]**2
    theta=np.arctan2(np.sqrt(yz),xyz[:,0])
    r_kf=np.sqrt(yz+xyz[:,0]**2)

    alpha=(0.58-0.007*Bz)*(1+0.024*np.log(Pd))
    #print(xyz_kf,R_mp,theta,alpha)
    b_mp=bl/(1+4*bl*(R_mp-R_bs))
    r_mp_kf=(-np.cos(theta)+np.sqrt(np.cos(theta)**2+4*R_mp*b_mp*np.sin(theta)**2))/(2*b_mp*np.sin(theta)**2)
    r_bs_kf=(-np.cos(theta)+np.sqrt(np.cos(theta)**2+4*R_bs*bl*np.sin(theta)**2))/(2*bl*np.sin(theta)**2)
    f_kf=(r_kf-r_mp_kf)/(r_bs_kf-r_mp_kf)
    #print('r_kf,r_mp,r_bs_kf',r_kf,r_mp,r_bs_kf)


    Cl=R_mpl/2*(2*R_bsl-R_mpl)/(R_bsl-R_mpl)
    v_fl=[]
    #xerR_mpl=R_mp-df
    check=0
    for i,vx,vy,vz,f,R_mp,R_bs,C,b,xf,R_bs_r in zip(xyz,v_x,v_y,v_z,f_msh,R_mpl,R_bsl,Cl,bl,xfl,R_bs):

#        print(i)
        x,y,z=i
        vsw=np.array([vx,vy,vz])
        d=np.sqrt((x-xf)**2+y**2+z**2)
        v0=np.array([C*(1/(2*d)-1/R_mp),C*y/(2*d*(d+x-xf)),C*z/(2*d*(d+x-xf))])
        v0=normalize(v0)
#        print(v0)
        #print(f,f_kf)
#        if (f_kf<0) or (f_kf>1):
#            v_f=np.array([np.nan,np.nan,np.nan])
#        else:
        while(x-R_bs_r+b*(y**2+z**2))<0:
            #print(x,y,z)
            #print(x-R_bs+1/(4*R_bs)*(y**2+z**2))
            d=np.sqrt((x-xf)**2+y**2+z**2)
            v=np.array([C*(1/(2*d)-1/R_mp),C*y/(2*d*(d+x-xf)),C*z/(2*d*(d+x-xf))])
            #print(v)
            x-=0.01*v[0]
            y-=0.01*v[1]
            z-=0.01*v[2]
        theta=np.arctan2(np.sqrt(y**2+z**2),x)
        phi=np.arctan2(z,y)
        if not 'v' in locals():
            check=1
            v_f=np.array([np.nan,np.nan,np.nan])
        else:
            #print(v)
            v=normalize(v)
            #t1=np.array([0,-np.sin(phi),np.cos(phi)])
            #t2=np.array([-np.sin(theta),np.cos(phi)*(1+np.cos(theta)),np.sin(phi)*(1+np.cos(theta))])
            #t2=normalize(t2)
#            t1=normalize(np.array([-2*b*z,0,1]))
#            t2=normalize(np.array([2*b*y,-(1+4*b**2*z**2),4*b**2*y*z]))
            n=normalize(np.array([1,2*b*y,2*b*z]))
            t1=normalize(v-np.inner(n,v)*n)
#            t2=normalize(np.cross(n,t1))
            #print(x,y,z)
            #print(vsw,v,t1,t2)
            r1=np.dot(vsw,t1)/np.dot(v,t1)
#            r2=np.dot(vsw,t2)/np.dot(v,t2)
#            r=np.sqrt((r1**2+r2**2)/2)
            #r=np.min([r1,r2])
            r=r1 #Quadratic means in Soucek 2012 can lead to blow up
            #print(r)
            rho=0.8+0.2*np.tanh(4*f)
            vm0=r/rho
            #print(r1,r2,r,rho,vm0)
            v_f=vm0*v0
    #    print(v_f)
        v_fl.append(v_f)

    v_fl=np.array(v_fl)
    if check:print('Outside of BS from the start for at least one datapoint')

    return v_fl



def OMNIread2(f):

    BErr='nan'          # err of B field
    VErr='nan'          # err of Velocity
    NErr='nan'               # err of Number density

    time,Bx,By,Bz,vx,vy,vz,n,Pd,Ma,Mm=[],[],[],[],[],[],[],[],[],[],[]
    header1,header2=[],[]
    list=glob.glob(f)
    f0=open(list[0],'r')
    for line in f0:
        line=line.strip()
        w=line.split()
        if w[4]!=BErr and w[7]!=VErr and w[8]!=NErr:
            time.append(datetime.datetime(int(w[0]),1,1)+datetime.timedelta(days=int(w[1])-1,hours=int(w[2]),minutes=int(w[3])))
            Bx.append(float(w[4]))
            By.append(float(w[5]))
            Bz.append(float(w[6]))
            vx.append(float(w[7]))
            vy.append(float(w[8]))
            vz.append(float(w[9]))
            n.append(float(w[10]))
            Pd.append(float(w[11]))
            Ma.append(float(w[12]))
            Mm.append(float(w[13]))
    Bx=np.array(Bx)
    By=np.array(By)
    Bz=np.array(Bz)
    vx=np.array(vx)
    vy=np.array(vy)
    vz=np.array(vz)
    n=np.array(n)
    Pd=np.array(Pd)
    Ma=np.array(Ma)
    Mm=np.array(Mm)
    B_sw=np.sqrt(Bx**2+By**2+Bz**2)
    V=np.sqrt(vx**2+vy**2+vz**2)
    omni=pd.DataFrame({'datetime':time,'Bx':Bx,'By':By,'Bz':Bz,'V':V,'vx':vx,'vy':vy,'vz':vz,'n':n,'Pd':Pd,'Ma':Ma,'Mm':Mm,'B':B_sw})
    return omni

def SSCread(f):
    dateparse = lambda x, y: pd.to_datetime(x + ' ' + y, format='%y/%m/%d %H:%M:%S')
    sat = pd.read_csv(f, header=None, sep='\s+', names=['d','t','x','y','z'])
    sat['datetime'] = pd.to_datetime(sat['d'] + ' ' + sat['t'], format='%y/%m/%d %H:%M:%S')
    sat = sat.drop(['d', 't'], axis=1)
    return sat


def main(xyz,f_sw,fout,mpdo=0,bsdo=0,model='jel'):
    mpdo=float(mpdo)
    bsdo=float(bsdo)

    if type(xyz)==str:
        xyz=SSCread(xyz)

    omni=OMNIread2(f_sw)
    if len(omni)==1:
        omni=pd.concat([omni]*np.shape(xyz)[0])

    if type(xyz)!=np.ndarray:
        test=xyz.merge(omni,how='outer')[['datetime','x','y','z']].sort_values(by=['datetime']).set_index('datetime').interpolate('index')
        omni=omni.set_index('datetime').join(test,how='left')
        xyz_np=omni[['x','y','z']].to_numpy()
    else:
        xyz_np=xyz

    xyz_np_0=xyz_np.copy()
    rm=R.from_euler('z',-4,degrees=True)
    xyz_np=(rm.as_matrix()@xyz_np.T).T

    #To calculate the flow velocity at a given point r in the magnetosheath, we proceed as follows:
    #(i) The MIPM coordinates (F ,ϑ,ϕ) are calculated using Eqs. (1) and (7) with the chosen models for the bow shock and the magnetopause, in our case Farris et al. (1991) and Shue et al. (1997).
    r=xyz_np[:,0]**2+xyz_np[:,1]**2+xyz_np[:,2]**2
    theta=np.arccos(xyz_np[:,0]/r)
    phi=np.arctan2(xyz_np[:,2],xyz_np[:,1])
    mpd=np.array(mpdf(omni.Pd,omni.Bz,xyz_np))
    mpd+=mpdo
    if model!='jer':
        bsd=bsdf_Jelinek(omni.Pd,xyz_np)
    else:
         bsd=np.array(BSf_Farris_Jerab(omni.Pd,omni.n,omni.V,omni.B,omni.Ma,xyz_np))
    bsd+=bsdo
    pts=appendSpherical_np(xyz_np)


    f_msh=(pts[:,3]-mpd)/(bsd-mpd)
    #print(mpd,bsd,f_msh)
    pts=np.hstack((pts,f_msh.reshape(pts.shape[0],1)))

    pts_df=pd.DataFrame(pts, columns=['x','y','z','r','theta','phi','f'])
    input=pts_df[['theta','phi','f']].to_numpy()

    #(ii) We calculate the corresponding KF94R point r˜ with the same MIPM coordinates, but using r˜bs and r˜mp from the KF94R model in Eq. (7).
    xyz_KF=KF94R(omni.Pd,omni.Bz,omni.n,omni.V,omni.B,omni.Ma,xyz_np,f_msh)
    #print(xyz_np,xyz_KF)


    #(iii) From r˜ is calculated the velocity vector v˜ in the KF94R reference frame using Eq. (3).
    v_t=backtrack(omni.Pd,omni.Bz,omni.n,omni.B,omni.Ma,omni.vx,omni.vy,omni.vz,xyz_KF,f_msh)
#    print('v_t',v_t)

    #(iv) The velocity vector v˜ is transformed back from the KF94R magnetosheath to the original GSE reference frame. Choose a small time increment ∆t and calculating the position of an adjacent point on the same flowline r˜0 = r˜ + v˜ ∆t. The point r˜0 is then easily transformed from the KF94R reference frame to the original GSE frame in a manner analogous to steps 1 and 2 above (let r0 be the resulting advanced GSE point). The resulting flow velocity vector in GSE coordinates is obtained as v = (r0 − r)/ ∆t.
    dt=0.01
#    print(np.shape(xyz_KF)[0])
#    print(np.shape(v_t))
#    print(v_t[0])
    r_t0=v_t*dt+xyz_KF
#    print(xyz_KF,r_t0)
    r0=KF94R_R(omni.Pd,omni.Bz,omni.n,omni.V,omni.B,omni.Ma,r_t0)
#    xyz_R=KF94R_R(omni.Pd,omni.Bz,omni.n,omni.V,omni.B,omni.Ma,xyz_KF)
#    print(r0,xyz_R)
    #print(r0-xyz_np)
    v=(r0-xyz_np)/dt
    v=(rm.as_matrix().T@v.T).T
    inv=np.logical_or(pts_df.f<0.05,pts_df.f>1)
    inv2=np.logical_or(np.absolute(v[:,0])>1000,np.absolute(v[:,1])>1000,np.absolute(v[:,2])>1000)
    v[inv]=np.array([np.nan,np.nan,np.nan])
    #v[inv2]=np.array([np.nan,np.nan,np.nan])


    out=pd.DataFrame({'time':omni.index,'x':xyz_np_0[:,0],'y':xyz_np_0[:,1],'z':xyz_np_0[:,2],'f':pts_df.f,'vx':v[:,0],'vy':v[:,1],'vz':v[:,2]})
    out.to_csv(fout,float_format='%.3f',index=False)
    return

if __name__=='__main__':
    f_sw="Soucek_test.txt"
    test=open(f_sw,"w")
    test.write("2003 124 11 34    2.17   -4.01    0.58  -407.2   -47.2     5.1   4.39  1.48   9.1  6.0\n")
    #Year DOY HR MN Bx By Bz Vx Vy Vz n Pd Ma Mm, as in OMNIweb
    #time is not used in the calculation
    #One line data can be used for constant SW cond.
    test.close()
#    f_sw='/Volumes/easystore/openggcm_run/Msh_Nstep/case_studies/2003May04_001/omni.lst'
    xyz=np.array([[  7.43,-7.58,6.56]])

#    xyz='/Volumes/easystore/openggcm_run/Msh_Nstep/case_studies/2003May04_001/cl_orbit.txt'

    #Orbit data from the SSWweb can be used as xyz. (filepath)
    #Data would be interpolated based on OMNI time.
    fout='/Volumes/easystore/openggcm_run/Msh_Nstep/case_studies/2003May04_001/Soucek_test.txt'
#    fout='/Volumes/easystore/openggcm_run/Msh_Nstep/case_studies/2003May04_001/Soucek_out.txt'
    main(xyz,f_sw,fout)



import numpy as np
import matplotlib.pyplot as plt
import pandas as pd


a=pd.read_csv('/Volumes/easystore/openggcm_run/Msh_Nstep/modeling/SMILE_002400')
fig=plt.figure()
ax=fig.add_subplot(projection='polar')
ax.set_thetamin(-90)
ax.set_thetamax(90)
ax.set_ylim([0,25])
im=ax.scatter(a.phi*np.pi/180,a.rad,c=a.ni_T,cmap='jet')
cb=fig.colorbar(im,ax=ax)
cb.set_label('n [cm$^{-3}$]')
b=a[a.f==0]
ax.plot(b.phi*np.pi/180,b.rad,label='MP')
b=a[a.f==1]
ax.plot(b.phi*np.pi/180,b.rad,label='BS')
ax.set_title('B>0 $n_{sw}=5cm^{-3}$')

plt.savefig('SMILE_fig_5.png')



"""A simplified magnetosheath model"""

__version__='1.0.0'
import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
from sys import argv
from .Msh_Nstep_test_n import main as mhd
from .Msh_Nstep_Soucek import main as Soucek
from .Msh_Nstep_Romashets import main as Romashets
from .Msh_Nstep_Spreiter_test import main as Spreiter
from .omni import main as omni
from .sscweb import main as sscweb
from .sc_data import main as sc_data

def msh_param(path,ts,te,sc,mpoff=0,bsoff=0):
    """Generate models result for given SW/IMF conditions & virtual spacecraft trace

    MHD-based model data files (*.npy) should be in /modeling/cn and /modeling/cs directory.

    Args:
        path: File directory. 
        ts: start time, in the format: '%Y-%m-%dT%H:%M'
        te: end time
        sc: spacecraft name for downloading orbit and observed data. See sc_data.py.
        mpoff: Optional; Manual magnetopause offset along x axis.
            Magnetopause model for the MHD-based MSH model is Shue et al. 1998.
        bsoff: Optional; Manual bow shock offset along x axis
            Bow shock model for the MHD-based MSH model is Jelinek et al. 2012.

    Returns:
        Following files are saved in the path.
            1. Msh_MHD_out.txt: MHD-based model result.
            2. Msh_Soucek_out.txt: Soucek & Escoubet [2012] model plasma velocity results.
            3. Msh_Romashtes_out.txt: Romashets et al. [2019] model magnetic field results.
            4. Msh_Spreiter_out.txt: Spreiter et al. [1966] model plasma speed, density, and temperature results.
    """

    omni(path,ts,te)
    f_sw=path+'/omni.lst'
    #SW/IMF conditions file
    sscweb(path,ts,te,sc)
    f_xyz=path+'/orbit.txt'
    #sc trace file
    fout=path+'/Msh_MHD_out.txt'
    #MHD-based model output file
    fout_v=path+'/Msh_Soucek_out.txt'
    #Soucek model velocity output file
    fout_b=path+'/Msh_Romashets_out.txt'
    #Romashets model magnetic field output file
    fout_n=path+'/Msh_Spreiter_out.txt'
    #Spreiter plasma density & temperature output file
    foff=path+'/Msh_offset.txt'

    mhd(f_xyz,f_sw,fout,mpoff,bsoff,foff,'jel')
    print('MHD-based model result calculated\n')
    Spreiter(f_xyz,f_sw,fout_n,mpoff,bsoff,'jel')
    print('Spreiter model n,T result calculated\n')
    Soucek(f_xyz,f_sw,fout_v,mpoff,bsoff,'jel')
    print('Soucek & Escoubet 2012 velocity model result calculated\n')
    Romashets(f_xyz,f_sw,fout_b,mpoff,bsoff,'jel')
    print('Romashets & Vandas 2019 magnetic field result calculated\n')
    return

if __name__=='__main__':
    msh_param(*argv[1:])



import numpy as np
#from mayavi import mlab
import matplotlib.pyplot as plt
import copy


theta, phi = np.linspace(0, np.pi, 40), np.linspace(0, 2*np.pi, 40)
THETA, PHI = np.meshgrid(theta, phi)

#mlab.clf()
x, y, z = np.mgrid[-40:40:50j, -40:40:50j, -40:40:50j]
x2=z2=np.arange(-40,40,.1)
X,Z=np.meshgrid(x2,z2)


# Jerab Bow Shock
a11=.45
a22=1
a33=.8
a12=.18
a14=46.6
a24=-2.2
a34=-.6
a44=-618
R0=11.8954

#Jelinek Bow Shock
R_0=15.02
lam=1.17
epsilon=6.55

#Jerab [2005] BS model
def bsy(ax,N,V,Ma,B,j):
    D=0.937*(0.846+0.042*B)
#    print(N,V,D,Ma)
    f=91.55/(N*V**2)**(1/6)*(1+D*(5/3-1)*(Ma**2+2)/((5/3+1)*(Ma**2-1)))/R0
#    values=a11*(x/f)**2+a22*(y/f)**2+a33*(z/f)**2+a12*(x/f)*(y/f)+a14*(x/f)+a24*(y/f)+a34*(z/f)+a44
    values2=a11*(X/f)**2+a22*(Z/f)**2+a12*(X/f)*(Z/f)+a14*(X/f)+a24*(Z/f)+a44
    if j==0: ln=ax.contour(X,Z,values2,0,colors='orange')
    else: ln=ax.contour(X,Z,values2,0,colors='orange',linestyles='dashed')
    ax.set_xlabel('X')
    ax.set_ylabel('Y')
    return ln

def bsz(ax,N,V,Ma,B,j):
    D=0.937*(0.846+0.042*B)
    f=91.55/(N*V**2)**(1/6)*(1+D*(5/3-1)*(Ma**2+2)/((5/3+1)*(Ma**2-1)))/R0
#    values=a11*(x/f)**2+a22*(y/f)**2+a33*(z/f)**2+a12*(x/f)*(y/f)+a14*(x/f)+a24*(y/f)+a34*(z/f)+a44
    values2=a11*(X/f)**2+a33*(Z/f)**2+a14*(X/f)+a34*(Z/f)+a44
    if j==0: ln=ax.contour(X,Z,values2,0,colors='orange')
    else: ln=ax.contour(X,Z,values2,0,colors='orange',linestyles='dashed')
    ax.set_xlabel('X')
    ax.set_ylabel('Z')
    return ln


#Jelinek [2012] BS model
def bsj(ax,Pd,j,k):
    a14=4*R_0/lam**2*Pd**(-1/epsilon)
    a44=-1/4*(lam*a14)**2
    values=Z**2+a14*X+a44
    if j==0: ln=ax.contour(X,Z,values,0,colors='orange')
    else: ln=ax.contour(X,Z,values,0,colors='orange',linestyles='dashed')
    ax.set_xlabel('X')
    if k=='y':
        ax.set_ylabel('Y')
    elif k=='z':
        ax.set_ylabel('Z')
    return ln




# Shue MP model
def mp2(ax,Pd,Bz,j):
    r0=(10.22+1.29*np.tanh(0.184*(Bz+8.14)))*Pd**(-1/6.6)
    alpha=(0.58-0.007*Bz)*(1+0.024*np.log(Pd))
    R = r0*(2/(1+np.cos(theta)))**alpha
    Z = R * np.sin(theta)
    Z2= -R*np.sin(theta)
    X = R * np.cos(theta)
    if j==0:
        ln,=ax.plot(X,Z,color='red')
        ln2,=ax.plot(X,Z2,color='red')
    else:
        ln,=ax.plot(X,Z,color='red',linestyle='dashed')
        ln2,=ax.plot(X,Z2,color='red',linestyle='dashed')
    ax.set_xlim(-25,25)
    ax.set_ylim(-25,25)
    return ln,ln2

B,Bz,V,N,Pd,Ma=4.78, -0.41, 511.3, 2.68, 1.4, 8.8
Earth=plt.Circle((0,0),1)
#Earth2=plt.Circle((0,0),1)


def BSMP(ax1,ax2,B,Bz,V,N,Pd,Ma,j):
    if j==0:
        ln1=bsy(ax1,N,V,Ma,B,0)
        ln2=mp2(ax1,Pd,Bz,0)
        ln3=ax1.add_patch(copy.copy(Earth))
        ln4=bsz(ax2,N,V,Ma,B,0)
        ln5=mp2(ax2,Pd,Bz,0)
        ln6=ax2.add_patch(copy.copy(Earth))
        #plt.tight_layout()
    else:
        ln1=bsy(ax1,N,V,Ma,B,1)
        ln2=mp2(ax1,Pd,Bz,1)
        ln3=ax1.add_patch(copy.copy(Earth))
        ln4=bsz(ax2,N,V,Ma,B,1)
        ln5=mp2(ax2,Pd,Bz,1)
        ln6=ax2.add_patch(copy.copy(Earth))
        #plt.tight_layout()
    return ln1,ln2,ln3,ln4,ln6,ln6

def BSMPj(ax1,ax2,Bz,Pd,j):
    if j==0:
        ln1=bsj(ax1,Pd,0,'y')
        ln2=mp2(ax1,Pd,Bz,0)
        ln3=ax1.add_patch(copy.copy(Earth))
        ln4=bsj(ax2,Pd,0,'z')
        ln5=mp2(ax2,Pd,Bz,0)
        ln6=ax2.add_patch(copy.copy(Earth))
        #plt.tight_layout()
    else:
        ln1=bsj(ax1,Pd,1,'y')
        ln2=mp2(ax1,Pd,Bz,1)
        ln3=ax1.add_patch(copy.copy(Earth))
        ln4=bsj(ax2,Pd,1,'z')
        ln5=mp2(ax2,Pd,Bz,1)
        ln6=ax2.add_patch(copy.copy(Earth))
        #plt.tight_layout()
    return ln1,ln2,ln3,ln4,ln6,ln6



if __name__=='__main__':
    #test(1,B,Bz,V,N,Pd,Ma,1)
    f,(ax1,ax2)=plt.subplots(1,2)
    BSMP(ax1,ax2,B,Bz,V,N,Pd,Ma,0)



import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
from scipy import stats
import sys

n=6
run=sys.argv[1]
lf=int(sys.argv[2])
if run=='Msh_Nstep':external='easystore'
else:external='Elements'

df_list2=[]
for i in range(1,n+1):
    df_list=[]
    for phi in np.arange(-90+7.5,90,7.5):
        a=pd.read_csv('/Volumes/'+external+'/openggcm_run/'+run+'/modeling/comp_phi_%.1f_' % phi+str(1200+1200*i).zfill(6))
        if lf==1:a=a[(a.f>=.3) & (a.f<=.7)]
        df_list.append(a)
    a2=pd.concat(df_list)
    a2['n']=i*5
    df_list2.append(a2)
a3=pd.concat(df_list2)
stat=pd.DataFrame([[round(a3.B.mean(),3),round(a3.ni.mean(),3),round(a3.Ti.mean(),3),round(a3.V.mean(),3)],
[round(a3.B_T.mean(),3),round(a3.ni_T.mean(),3),round(a3.Ti_T.mean(),3),round(a3.V_T.mean(),3)],
[a3.B.median(),a3.ni.median(),a3.Ti.median(),a3.V.median()],
[a3.B_T.median(),a3.ni_T.median(),a3.Ti_T.median(),a3.V_T.median()]])
if lf!=1:stat.to_csv('/Volumes/'+external+'/openggcm_run/'+run+'/modeling/stat.txt')
else:stat.to_csv('/Volumes/'+external+'/openggcm_run/'+run+'/modeling/stat_lf.txt')


list=['B','ni','Ti','V']
unit=['[nT]','[cm$^{-3}$]','[eV]','[km/s]']
reg=[]
for j in range(4):
    mask=~np.isnan(a3[list[j]+'_T'])&~np.isnan(a3[list[j]])
    reg.append(stats.linregress(a3[list[j]+'_T'][mask],a3[list[j]][mask]))

label_l=np.arange(5,35,5)
fig,ax=plt.subplots(2,2,figsize=(10,10))
for j in range(4):
    ax_t=ax.flatten()[j]
    for i in range(n):
        ax_t.scatter(df_list2[i][list[j]+'_T'],df_list2[i][list[j]],label=str(label_l[i])+'cm$^{-3}$')
    ax_t.plot(a3[list[j]+'_T'],reg[j].slope*a3[list[j]+'_T']+reg[j].intercept)
    ax_t.annotate('y={:.2f}x+{:.2f}'.format(reg[j].slope,reg[j].intercept),xy=(.05,.95),xycoords='axes fraction')
    ax_t.plot(a3[list[j]+'_T'],a3[list[j]+'_T'],label='y=x')
    ax_t.grid()
#    ax_t.legend()
    ax_t.set_xlabel(list[j]+'_THEMIS '+unit[j])
    ax_t.set_ylabel(list[j]+'_MHD '+unit[j])
handles, labels = ax_t.get_legend_handles_labels()
fig.legend(handles, labels, loc='upper right',bbox_to_anchor=(0.9,0.99))
#plt.suptitle('B_z=-4nT, 0.3$\leq$f$\leq$0.7')
if run=='Msh_Nstep':plt.suptitle('B_z=4nT')
else:plt.suptitle('B_z=-4nT')

if lf!=1:plt.savefig('/Volumes/'+external+'/openggcm_run/'+run+'/modeling/plots/comp/Scatter.png')
else:plt.savefig('/Volumes/'+external+'/openggcm_run/'+run+'/modeling/plots/comp/Scatter_lf.png')



#Romashets Magnetic field

import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
from scipy.special import jv,iv,kn,jvp,ivp,kvp
import glob
import datetime
from scipy.spatial.transform import Rotation as R

# Bow Shock
a11=.45
a22=1
a33=.8
a12=.18
a14=46.6
a24=-2.2
a34=-.6
a44=-618
R0=11.8954

R_0=15.02
lam=1.17
epsilon=6.55

def appendSpherical_np(xyz):
    ptsnew = np.hstack((xyz,np.empty(xyz.shape)))
    xy = xyz[:,0]**2 + xyz[:,1]**2
    ptsnew[:,3] = np.sqrt(xy + xyz[:,2]**2)
    #ptsnew[:,4] = np.arctan2(np.sqrt(xy), xyz[:,2]) # for elevation angle defined from Z-axis down
    ptsnew[:,4] = np.arctan2(xyz[:,2], np.sqrt(xy)) # for elevation angle defined from XY-plane up
    ptsnew[:,5] = np.arctan2(xyz[:,1], xyz[:,0])
    return ptsnew

def bsdf2(N,V,Ma,B,xl,yl,zl):
    D=0.937*(0.846+0.042*B)
    f=91.55/(N*V**2)**(1/6)*(1+D*((5/3-1)*Ma**2+2)/((5/3+1)*(Ma**2-1)))/R0
    t=0
    sol0=a44
    sol=sol0
    while sol*sol0>0:
        t+=0.001
        x=xl*t
        y=yl*t
        z=zl*t
        sol=a11*(x/f)**2+a22*(y/f)**2+a33*(z/f)**2+a12*(x/f)*(y/f)+a14*(x/f)+a24*(y/f)+a34*(z/f)+a44
#        print(sol)
    dist=np.linalg.norm(np.array([xl*t,yl*t,zl*t]))
    return dist

def bsdf_Jelinek(Pd,xyz):

    a_14=(4*R_0/lam**2*Pd**(-1/epsilon))
    a_44=(-1/4*lam**2*a_14**2)
    yz=xyz[:,1]**2+xyz[:,2]**2
    theta=np.arctan2(np.sqrt(yz),xyz[:,0])
    cos=np.cos(theta)
    sin=np.sin(theta)
    r=np.array((-a_14*cos+np.sqrt(a_14**2*cos**2-4*a_44*sin**2))/(2*sin**2))
    r0=-a_44/a_14
    r[np.isnan(r)]=r0[np.isnan(r)]
    return r


def mpdf(Pd,Bz,xyz):
    yz=xyz[:,1]**2+xyz[:,2]**2
    theta=np.arctan2(np.sqrt(yz),xyz[:,0])
    r0=(10.22+1.29*np.tanh(0.184*(Bz+8.14)))*Pd**(-1/6.6)
    alpha=(0.58-0.007*Bz)*(1+0.024*np.log(Pd))
    R = r0*(2/(1+np.cos(theta)))**alpha
    return R

def P(v,k,s,t,s0):
    if v==0:
        f=iv(1,k*s)-kn(1,k*s)*ivp(1,k*s0)/ivp(1,k*s0)
    if v==1:
        f=(ivp(0,k*s)+kvp(0,k*s)*iv(1,k*s0)/kn(1,k*s0))*jv(0,k*t)
    if v==2:
        f=(iv(0,k*s)+kn(0,k*s)*iv(1,k*s0)/kn(1,k*s0))*jvp(0,k*t)
    if v==3:
        f=(ivp(1,k*s)-kvp(1,k*s)*ivp(1,k*s0)/kvp(1,k*s0))*jv(1,k*t)
    if v==4:
        f=(iv(1,k*s)-kn(1,k*s)*ivp(1,k*s0)/ivp(1,k*s0))*jvp(1,k*t)
    return f

"""
A,B=np.hsplit(np.array([
       [ 1.40701261e+03,  3.04789295e+03],
       [ 4.10907593e+00,  1.19842529e+01],
       [ 4.16965516e-01,  1.07833018e+00],
       [ 4.43112000e-02,  1.07371444e-01],
       [ 5.12188800e-03,  1.16881590e-02],
       [ 5.81770000e-04,  1.30437000e-03],
       [ 6.98000000e-05,  1.51000000e-04],
       [ 8.05000000e-06,  1.75000000e-05],
       [ 9.94000000e-07,  2.07000000e-06],
       [ 1.14000000e-07,  2.44000000e-07],
       [ 1.45000000e-08,  2.93000000e-08],
       [ 1.61000000e-09,  3.49000000e-09],
       [ 2.18000000e-10,  4.22000000e-10],
       [ 2.27000000e-11,  5.06000000e-11],
       [ 3.37000000e-12,  6.15000000e-12],
       [ 3.08000000e-13,  7.41000000e-13],
       [ 5.27000000e-14,  8.87000000e-14],
       [ 3.59000000e-15,  1.06000000e-14],
       [ 6.82000000e-16,  1.10000000e-15],
       [-6.13000000e-18,  9.94000000e-17]]),2)
A=A.flatten()
B=B.flatten()
"""

def OMNIread2(f):

    BErr='nan'          # err of B field
    VErr='nan'          # err of Velocity
    NErr='nan'               # err of Number density

    time,Bx,By,Bz,vx,vy,vz,n,Pd,Ma,Mm=[],[],[],[],[],[],[],[],[],[],[]
    header1,header2=[],[]
    list=glob.glob(f)
    f0=open(list[0],'r')
    for line in f0:
        line=line.strip()
        w=line.split()
        if w[4]!=BErr and w[7]!=VErr and w[8]!=NErr:
            time.append(datetime.datetime(int(w[0]),1,1)+datetime.timedelta(days=int(w[1])-1,hours=int(w[2]),minutes=int(w[3])))
            Bx.append(float(w[4]))
            By.append(float(w[5]))
            Bz.append(float(w[6]))
            vx.append(float(w[7]))
            vy.append(float(w[8]))
            vz.append(float(w[9]))
            n.append(float(w[10]))
            Pd.append(float(w[11]))
            Ma.append(float(w[12]))
            Mm.append(float(w[13]))
    Bx=np.array(Bx)
    By=np.array(By)
    Bz=np.array(Bz)
    vx=np.array(vx)
    vy=np.array(vy)
    vz=np.array(vz)
    n=np.array(n)
    Pd=np.array(Pd)
    Ma=np.array(Ma)
    Mm=np.array(Mm)
    B_sw=np.sqrt(Bx**2+By**2+Bz**2)
    V=np.sqrt(vx**2+vy**2+vz**2)
    omni=pd.DataFrame({'datetime':time,'Bx':Bx,'By':By,'Bz':Bz,'V':V,'vx':vx,'vy':vy,'vz':vz,'n':n,'Pd':Pd,'Ma':Ma,'Mm':Mm,'B':B_sw})
    return omni

def SSCread(f):
    dateparse = lambda x, y: pd.to_datetime(x + ' ' + y, format='%y/%m/%d %H:%M:%S')
    sat = pd.read_csv(f, header=None, sep='\s+', names=['d','t','x','y','z'])
    sat['datetime'] = pd.to_datetime(sat['d'] + ' ' + sat['t'], format='%y/%m/%d %H:%M:%S')
    sat = sat.drop(['d', 't'], axis=1)
    return sat



def main(xyz,f_sw,fout,mpdo=0,bsdo=0,model='jel'):
    mpdo=float(mpdo)
    bsdo=float(bsdo)

    if type(xyz)==str:
        xyz=SSCread(xyz)

    omni=OMNIread2(f_sw)
    if len(omni)==1:
        omni=pd.concat([omni]*np.shape(xyz)[0])

    if type(xyz)!=np.ndarray:
        test=xyz.merge(omni,how='outer')[['datetime','x','y','z']].sort_values(by=['datetime']).set_index('datetime').interpolate('index')
        omni=omni.set_index('datetime').join(test,how='left')
        xyz_np=omni[['x','y','z']].to_numpy()
    else:
        xyz_np=xyz

    rm=R.from_euler('z',-4,degrees=True)
    xyz_np=(rm.as_matrix()@xyz_np.T).T

    mpd=np.array(mpdf(omni.Pd,omni.Bz,xyz_np))
    mpd+=mpdo
#    bsd=np.array([bsdf(omni.n[i],v_sw[i],Ma[i],B_sw[i],xyz[i])for i in range(np.shape(xyz)[0])])

    if model!='jel':
        bsdfv=np.vectorize(bsdf2)
        bsd=np.array(bsdfv(omni.n,omni.V,omni.Ma,omni.B,xyz_np[:,0],xyz_np[:,1],xyz_np[:,2]))
    else:
        bsd=bsdf_Jelinek(omni.Pd,xyz_np)
    bsd+=bsdo
#    print(np.shape(mpd),np.shape(bsd))

    pts=appendSpherical_np(xyz_np)
    print('theta range',np.nanmin(pts[:,4])*180/np.pi,np.nanmax(pts[:,4])*180/np.pi)
    print('phi range',np.nanmin(pts[:,5]*180/np.pi),np.nanmax(pts[:,5])*180/np.pi)

    f_msh=(pts[:,3]-mpd)/(bsd-mpd)
#    print(type(pts),type(f_msh))
    pts=np.hstack((pts,f_msh.reshape(pts.shape[0],1)))
    #print(pts)
    pts_df=pd.DataFrame(pts, columns=['x','y','z','r','theta','phi','f'])

    #Finding Ai and Bi, using mid time data
    mid_rw = int(len(omni)/2)
    x,y,z=xyz_np[mid_rw]
    Pd=omni.Pd.iloc[mid_rw]
    Bx=omni.Bx.iloc[mid_rw]
    By=omni.By.iloc[mid_rw]
    Bz=omni.Bz.iloc[mid_rw]

    xc=12.82*Pd**(-1/5.26)*(1-1/1.54**2)
    #xs=-1.91
    xs=15.02*Pd**(-1/6.55)*(1-1/1.17**2)

    #s0=3.66
    s0=np.sqrt(2*12.82)/1.54*Pd**(-1/(2*5.26))
    sig1=np.sqrt(2*15.02)/1.17*Pd**(-1/(2*6.55))
    #C=2
    C=sig1**2/(sig1**2-s0**2)
    #print(xc,xs,s0,C)

    kmax=4.5
    check=150
    while (check>35):
        kmax+=0.5
        k=np.linspace(0.01,kmax,20)
        taul=np.arange(1,11)

        F_A,G_A,F_B,G_B=[],[],[],[]

        for j in range(0,10):
            tau=taul[j]
            h1=np.sqrt(sig1**2+tau**2)
            q1=(sig1**2-tau**2)/2-xc+xs
            s1=np.sqrt(q1+np.sqrt(q1**2+sig1**2*tau**2))
            t1=np.sqrt(-q1+np.sqrt(q1**2+sig1**2*tau**2))
            dsdsig1=sig1/(2*s1)*(1+(q1+tau**2)/np.sqrt(q1**2+sig1**2*tau**2))
            dtdsig1=-sig1/(2*t1)*(1-(q1+tau**2)/np.sqrt(q1**2+sig1**2*tau**2))
        #    print(t1,dtdsig1)
            G_A_temp=sig1*(1-C)/np.sqrt(sig1**2+tau**2)+C*(1/h1*s0**2/s1*dsdsig1)
            G_A.append(G_A_temp)
            G_B_temp=tau*(1-C)/np.sqrt(sig1**2+tau**2)-C*(1/h1*s0**2/s1*(dtdsig1-t1/s1*dsdsig1))
            G_B.append(G_B_temp)

            Fl_A_temp=[]
            Fl_B_temp=[]
            for i in range(20):
                sum_sig_1=k[i]*(P(1,k[i],s1,t1,s0)*dsdsig1+P(2,k[i],s1,t1,s0)*dtdsig1)
                F_A_temp=(C*(sig1-s0**2/s1*dsdsig1)+sum_sig_1)/h1
                Fl_A_temp.append(F_A_temp)
                sum_sig_2=k[i]*(P(3,k[i],s1,t1,s0)*dsdsig1+P(4,k[i],s1,t1,s0)*dtdsig1)
                F_B_temp=(C*(tau+s0**2/s1*(dtdsig1-t1/s1*dsdsig1))+sum_sig_2)/h1
                Fl_B_temp.append(F_B_temp)
            F_A.append(Fl_A_temp)
            F_B.append(Fl_B_temp)
        G_A=np.array(G_A)
        F_A=np.array(F_A)
        G_B=np.array(G_B)
        F_B=np.array(F_B)
        #print(np.shape(G),np.shape(F),np.shape(F.T@F))
        A=(np.linalg.inv(F_A.T@F_A)@F_A.T)@G_A
        B=(np.linalg.inv(F_B.T@F_B)@F_B.T)@G_B

        #print(A,B)
#        print('A,B calculated')

        Bxl,Byl,Bzl,B_tl=[],[],[],[]
        for (xyz,Pd,B0x,B0y,B0z,f) in zip(xyz_np,omni.Pd,omni.Bx,omni.By,omni.Bz,pts_df.f):
            if (0<=f) & (f<=1):
                x,y,z=xyz
                #xc=1.55
                xc=12.82*Pd**(-1/5.26)*(1-1/1.54**2)
                #xs=-1.91
                xs=15.02*Pd**(-1/6.55)*(1-1/1.17**2)
                phi=np.arctan2(z,y)
                r_s=np.sqrt((x-xs)**2+y**2+z**2)
                r_c=np.sqrt((x-xc)**2+y**2+z**2)
                sig=np.sqrt(r_s+x-xs)
                tau=np.sqrt(r_s-x+xs)
                s=np.sqrt(r_c+x-xc)
                t=np.sqrt(r_c-x+xc)
              #  s=np.sqrt(r_c+x-xc)
              #  t=np.sqrt(r_c-x+xc)
                q=(sig**2-tau**2)/2-xc+xs
        #        s=np.sqrt(q+np.sqrt(q**2+sig**2*tau**2))
        #        t=np.sqrt(-q+np.sqrt(q**2+sig**2*tau**2))

                h=np.sqrt(sig**2+tau**2)
                h_phi=sig*tau

                #s0=3.66
                s0=np.sqrt(2*12.82)/1.54*Pd**(-1/(2*5.26))
                sig1=np.sqrt(2*15.02)/1.17*Pd**(-1/(2*6.55))
                #C=2
                C=sig1**2/(sig1**2-s0**2)
                #print(xc,xs,s0,C)

                denom=np.sqrt(q**2+sig**2*tau**2)
                dsdsig=sig/(2*s)*(1+(q+tau**2)/denom)
                dsdtau=-tau/(2*s)*(1+(q-sig**2)/denom)
                dtdsig=-sig/(2*t)*(1-(q+tau**2)/denom)
                dtdtau=tau/(2*t)*(1-(q-sig**2)/denom)
                #print(denom,dsdsig,dsdtau,dtdsig,dtdtau)

             #   k=np.linspace(0.01,5.5,20)

                sum_sig_1,sum_sig_2,sum_tau_1,sum_tau_2,sum_phi=0,0,0,0,0
                for i in range(20):
            #        print('A',i,A[i]*k[i]*(P(1,k[i],s,t,s0)*dsdsig+P(2,k[i],s,t,s0)*dtdsig))
            #        print('B',i,B[i]*k[i]*(P(3,k[i],s,t,s0)*dsdsig+P(4,k[i],s,t,s0)*dtdsig))
                    sum_sig_1+=A[i]*k[i]*(P(1,k[i],s,t,s0)*dsdsig+P(2,k[i],s,t,s0)*dtdsig)
                    sum_sig_2+=B[i]*k[i]*(P(3,k[i],s,t,s0)*dsdsig+P(4,k[i],s,t,s0)*dtdsig)
                    sum_tau_1+=A[i]*k[i]*(P(1,k[i],s,t,s0)*dsdtau+P(2,k[i],s,t,s0)*dtdtau)
                    sum_tau_2+=B[i]*k[i]*(P(3,k[i],s,t,s0)*dsdtau+P(4,k[i],s,t,s0)*dtdtau)
                    sum_phi+=B[i]*P(0,k[i],s,t,s0)*jv(1,k[i]*t)
            #    print(B0x,h,C,sig,s0,s,dtdsig,sum_sig_1,B0y,B0z,tau,dtdtau,sum_sig_2,A[i],B[i],k[i],phi)
            #    print(sum_sig_1,sum_sig_2,sum_tau_1,sum_tau_2,sum_phi)
                B_sig=(B0x*(C*(sig-s0**2/s*dsdsig)+sum_sig_1)+(B0y*np.cos(phi)+B0z*np.sin(phi))*(C*(tau+s0**2/s*(dtdsig-t/s*dsdsig))+sum_sig_2))/h
                B_tau=(B0x*(-C*(tau+s0**2/s*dsdtau)+sum_sig_1)+(B0y*np.cos(phi)+B0z*np.sin(phi))*(C*(sig+s0**2/s*(dtdtau-t/s*dsdtau))+sum_tau_2))/h
                B_phi=(-B0y*np.sin(phi)+B0z*np.cos(phi))/h_phi*(C*(sig*tau+s0**2*t/s)+sum_phi)
                #print(B_sig,B_tau,B_phi)

                Bx=(B_sig*sig-B_tau*tau)/h
                By=(B_sig*tau+B_tau*sig)/h*np.cos(phi)-B_phi*np.sin(phi)
                Bz=(B_sig*tau+B_tau*sig)/h*np.sin(phi)+B_phi*np.cos(phi)
                B_t=np.sqrt(Bx**2+By**2+Bz**2)
            else:
                Bx=np.nan
                By=np.nan
                Bz=np.nan
                B_t=np.nan
            Bxl.append(Bx)
            Byl.append(By)
            Bzl.append(Bz)
            B_tl.append(B_t)
        check=np.nanmax(B_tl)
        print(kmax,check)
    out=pd.DataFrame({'time':omni.index,'Bx':Bxl,'By':Byl,'Bz':Bzl,'B':B_tl})
    out.to_csv(fout,float_format='%.3f',index=False)
    return

if __name__=='__main__':
#    f_sw="Romashets_test.txt"
#    fout='/Volumes/easystore/openggcm_run/Msh_Nstep/case_studies/2003May04_001/Romashets_test.txt'
#    xyz=np.array([[10,2,3],[8,3,1]])

    f_sw='/Volumes/easystore/openggcm_run/Msh_Nstep/case_studies/2003May04_001/omni.lst'
    xyz='/Volumes/easystore/openggcm_run/Msh_Nstep/case_studies/2003May04_001/cl_orbit.txt'

    #Orbit data from the SSWweb can be used as xyz. (filepath)
    #Data would be interpolated based on OMNI time.
#    fout='/Volumes/easystore/openggcm_run/Msh_Nstep/case_studies/2003May04_001/Soucek_test.txt'
    fout='/Volumes/easystore/openggcm_run/Msh_Nstep/case_studies/2003May04_001/Romashets_out.txt'

    main(xyz,f_sw,fout)



#!/Users/jjung15/miniconda3/bin/python
# Calculate input for plot_rates.py
# Written by J.Jung


import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
import sys
from sympy.solvers import solve
from sympy import Symbol,nsolve
from sympy import sqrt,log,tanh
from scipy.interpolate import NearestNDInterpolator as NNDI
from scipy.interpolate import LinearNDInterpolator as LNDI
from scipy.interpolate import RegularGridInterpolator as RGI
from scipy.spatial.transform import Rotation as R
import glob
import datetime

def main(arg):

#    arg=arg_str.split()

        
    # Bow Shock
    a11=.45
    a22=1
    a33=.8
    a12=.18
    a14=46.6
    a24=-2.2
    a34=-.6
    a44=-618
    R0=11.8954

    def bsdf(D,f,pts):

        sol0=a44
        bsd=[]
    #    print(pts[4])
        dx=0.1*np.cos(pts[:,4])*np.cos(pts[:,5])
        dy=0.1*np.cos(pts[:,4])*np.sin(pts[:,5])
        dz=0.1*np.sin(pts[:,4])

        for dxi,dyi,dzi in zip(dx,dy,dz):
            sol=sol0
    #        print(sol,sol0)
            x=100*dxi
            y=100*dyi
            z=100*dzi
            while sol*sol0>0:
                x+=dxi
                y+=dyi
                z+=dzi
                sol=a11*(x/f)**2+a22*(y/f)**2+a33*(z/f)**2+a12*(x/f)*(y/f)+a14*(x/f)+a24*(y/f)+a34*(z/f)+a44
                #print(sol)
            dist=np.sqrt(x**2+y**2+z**2)
        bsd.append(dist)
        return bsd

    def bsdf_Jelinek(a_14,a_44,xyz):
        yz=xyz[:,1]**2+xyz[:,2]**2
        theta=np.arctan2(np.sqrt(yz),xyz[:,0])
        cos=np.cos(theta)
        sin=np.sin(theta)
        r=(-a_14*cos+np.sqrt(a_14**2*cos**2-4*a_44*sin**2))/(2*sin**2)
        r0=-a_44/a_14
        r[np.logical_or(np.isnan(r),r<1e-10)]=r0
        return r



    def mpdf(r0,alpha,xyz):
        yz=xyz[:,1]**2+xyz[:,2]**2
        theta=np.arctan2(np.sqrt(yz),xyz[:,0])
        R = r0[0]*(2/(1+np.cos(theta)))**alpha[0]
        return R

    def genf(fun,n1,n2,xyz,f,f2):
        outl=[]
        for i in xyz:
            if i[2]<0.1 or i[2]>0.9:
                out=np.nan
            else:
                if f2==-1:
                    out1=fun[n1+7](i)[0]
                    out2=fun[n2+7](i)[0]
                    out=(out1*(1-f)+out2*f)
                elif f2==2:
                    out1=fun[n1](i)[0]
                    out2=fun[n2](i)[0]
                    out=(out1*(1-f)+out2*f)
                else:
                    out1=fun[n1](i)[0]
                    out2=fun[n2](i)[0]
                    out=(out1*(1-f)+out2*f)
                    out1=fun[n1+7](i)[0]
                    out2=fun[n2+7](i)[0]
                    outs=(out1*(1-f)+out2*f)
                    out=f2*out+(1-f2)*outs
            outl.append(out)
        outl=np.array(outl)
        return outl

    nfl=[]
    Tfl=[]
    Vxfl=[]
    Vyfl=[]
    Vzfl=[]
    the=np.arange(-90,91,1)*np.pi/180
    phi=np.arange(-90,91,1)*np.pi/180
    af=np.arange(0,11,1)/10

    def OMNIread2(f):

        BErr='9999.99'          # err of B field
        VErr='99999.9'          # err of Velocity
        NErr='999.99'               # err of Number density

        time,Bx,By,Bz,vx,vy,vz,n,Pd,Ma,Mm=[],[],[],[],[],[],[],[],[],[],[]
        header1,header2=[],[]
        list=glob.glob(f)
        f0=open(list[0],'r')
        for line in f0:
            line=line.strip()
            w=line.split()
            if w[4]!=BErr and w[7]!=VErr and w[8]!=NErr:
                time.append(datetime.datetime(int(w[0]),1,1)+datetime.timedelta(days=int(w[1])-1,hours=int(w[2]),minutes=int(w[3])))
                Bx.append(float(w[4]))
                By.append(float(w[5]))
                Bz.append(float(w[6]))
                vx.append(float(w[7]))
                vy.append(float(w[8]))
                vz.append(float(w[9]))
                n.append(float(w[10]))
                Pd.append(float(w[11]))
                Ma.append(float(w[12]))
                Mm.append(float(w[13]))
        Bx=np.array(Bx)
        By=np.array(By)
        Bz=np.array(Bz)
        vx=np.array(vx)
        vy=np.array(vy)
        vz=np.array(vz)
        n=np.array(n)
        Pd=np.array(Pd)
        Ma=np.array(Ma)
        Mm=np.array(Mm)
        B_sw=np.sqrt(Bx**2+By**2+Bz**2)
        V=np.sqrt(vx**2+vy**2+vz**2)
        omni=pd.DataFrame({'datetime':time,'Bx':Bx,'By':By,'Bz':Bz,'V':V,'vx':vx,'vy':vy,'vz':vz,'n':n,'Pd':Pd,'Ma':Ma,'Mm':Mm,'B':B_sw})
        return omni

    def appendSpherical_np(xyz):
        ptsnew = np.hstack((xyz,np.empty(xyz.shape)))
        xy = xyz[:,0]**2 + xyz[:,1]**2
        ptsnew[:,3] = np.sqrt(xy + xyz[:,2]**2)
        #ptsnew[:,4] = np.arctan2(np.sqrt(xy), xyz[:,2]) # for elevation angle defined from Z-axis down
        ptsnew[:,4] = np.arctan2(xyz[:,2], np.sqrt(xy)) # for elevation angle defined from XY-plane up
        ptsnew[:,5] = np.arctan2(xyz[:,1], xyz[:,0])
        return ptsnew

    path=arg[2]
    xsc=float(arg[4])
    ysc=float(arg[6])
    zsc=float(arg[8])
    xlk=float(arg[10])
    ylk=float(arg[12])
    zlk=float(arg[14])
    the1=float(arg[16])
    the2=float(arg[18])
    dthe=float(arg[20])
    phi1=float(arg[22])
    phi2=float(arg[24])
    dphi=float(arg[26])
    fout=arg[28]
    model=arg[30]
    print(path,xsc,ysc,zsc,xlk,ylk,zlk,the1,the2,dthe,phi1,phi2,dphi,fout)

    for i in range(0,7):
        rr1=np.load(path+'/cn/Msh_n_%d.npy'%i)
        pp1=np.load(path+'/cn/Msh_pp_%d.npy'%i)
        T1=72429*pp1/rr1/11600
        vx1=np.load(path+'/cn/Msh_Vx_%d.npy'%i)
        vy1=np.load(path+'/cn/Msh_Vy_%d.npy'%i)
        vz1=np.load(path+'/cn/Msh_Vz_%d.npy'%i)

        nf=RGI((the,phi,af),rr1,bounds_error=False)
        Tf=RGI((the,phi,af),T1,bounds_error=False)
        Vxf=RGI((the,phi,af),vx1,bounds_error=False)
        Vyf=RGI((the,phi,af),vy1,bounds_error=False)
        Vzf=RGI((the,phi,af),vz1,bounds_error=False)
        nfl.append(nf)
        Tfl.append(Tf)
        Vxfl.append(Vxf)
        Vyfl.append(Vyf)
        Vzfl.append(Vzf)

    for i in range(0,7):
        rr1=np.load(path+'/cs/Msh_n_%d.npy'%i)
        pp1=np.load(path+'/cs/Msh_pp_%d.npy'%i)
        T1=72429*pp1/rr1/11600
        vx1=np.load(path+'/cs/Msh_Vx_%d.npy'%i)
        vy1=np.load(path+'/cs/Msh_Vy_%d.npy'%i)
        vz1=np.load(path+'/cs/Msh_Vz_%d.npy'%i)

        nf=RGI((the,phi,af),rr1,bounds_error=False)
        Tf=RGI((the,phi,af),T1,bounds_error=False)
        Vxf=RGI((the,phi,af),vx1,bounds_error=False)
        Vyf=RGI((the,phi,af),vy1,bounds_error=False)
        Vzf=RGI((the,phi,af),vz1,bounds_error=False)
        nfl.append(nf)
        Tfl.append(Tf)
        Vxfl.append(Vxf)
        Vyfl.append(Vyf)
        Vzfl.append(Vzf)



    nh0=25
    drad=0.2*6372*1e3
    ri=2.1
    str1=4*np.pi
    crs=6e-16



    distance=np.sqrt((xsc-xlk)**2+(ysc-ylk)**2+(zsc-zlk)**2)
    p=np.arctan2(ysc-ylk,xsc-xlk)
    t=np.arccos((zsc-zlk)/distance)
    print(distance,p*180/np.pi,t*180/np.pi)

    f_sw="SW_cond.txt"
#    test=open(f_sw,"w")
#    test.write("1967 1 0 00 2 -2 -5 -400 0 0 10 2 10 6\n")
    #Year DOY HR MN Bx By Bz Vx Vy Vz n Pd Ma Mm, as in OMNIweb
    test.close()
    omni=OMNIread2(f_sw)
    D=0.937*(0.846+0.042*omni.B)
    f=(91.55/(omni.n*omni.V**2)**(1/6)*(1+D*((5/3-1)*omni.Ma**2+2)/((5/3+1)*(omni.Ma**2-1)))/R0)[0]
    r0=(10.22+1.29*np.tanh(0.184*(omni.Bz+8.14)))*omni.Pd**(-1/6.6)
    alpha=(0.58-0.007*omni.Bz)*(1+0.024*np.log(omni.Pd))
    mp0=mpdf(r0,alpha,np.array([[1,0,0]]))
    bs0=bsdf(D,f,np.array([[0,0,0,1,0,0]]))
    print('Shue/Jerab',mp0,bs0)
    R_0=15.02
    lam=1.17
    epsilon=6.55
    a_14=(4*R_0/lam**2*omni.Pd**(-1/epsilon))[0]
    a_44=(-1/4*lam**2*a_14**2)
    bs0_j=bsdf_Jelinek(a_14,a_44,np.array([[1,0,0]]))
    print('a_14,a_44',a_14,a_44)
    print('Shue/Jelinek',mp0,bs0_j)


    #bsdfv=np.vectorize(bsdf)
    if (omni.n>35).any():print('Warning: Solar wind density goes above 35cm^-3, in which our model was not validated.')
    if (omni.B>15).any():print('Warning: IMF magnitude goes above 15cm^-3, in which our model shows discrepancy with THEMIS data.')

    conditions=[omni.n<=1,(1<omni.n) & (omni.n<=5),(5<omni.n) & (omni.n<10),(10<=omni.n) & (omni.n<15),(15<=omni.n) & (omni.n<20),(20<=omni.n) & (omni.n<25),(25<=omni.n) & (omni.n<30),(30<=omni.n) & (omni.n<35)]
    choices1=[0,0,1,2,3,4,5,6]
    choices2=[0,1,2,3,4,5,6,6]
    n1=np.select(conditions,choices1)[0]
    n2=np.select(conditions,choices2)[0]
    f1=np.where(n2==1,(omni.n-1)/4,omni.n/5-n1)[0]
    conditions2=[omni.Bz<=-4,(-4<omni.Bz) & (omni.Bz<4), 4<=omni.Bz]
    choices3=[-1,(omni.Bz+4)/8,2]
    f2=np.select(conditions2,choices3)[0]
    print('f1,f2',f1,f2)


    fo=open(fout,'w+')
    #fout2='test.txt'
    #fo2=open(fout2,'w+')
    fo.write('%d %d %f %f %f %f %f %f\n' % (((phi2-phi1)/dphi+1),((the2-the1)/dthe+1),xsc,ysc,zsc,xlk,ylk,zlk))
    thel=np.arange(the1,the2+dthe,dthe)
    phil=np.arange(phi1,phi2+dphi,dphi)
    for i,the in enumerate(thel):
        lat=90-the
        the_r=the*np.pi/180
        for j,phi in enumerate(phil):
            phi_r=phi*np.pi/180
            rate=0
            los=[]
            for r in np.arange(0,100+0.2,0.2):
                x1=r*np.sin(the_r)*np.cos(phi_r)
                y1=r*np.sin(the_r)*np.sin(phi_r)
                z1=r*np.cos(the_r)
                x2=x1*np.sin(t)*np.cos(p)-y1*np.sin(p)-z1*np.cos(t)*np.cos(p)
                y2=x1*np.sin(t)*np.sin(p)+y1*np.cos(p)-z1*np.cos(t)*np.sin(p)
                z2=x1*np.cos(t)+z1*np.sin(t)
                x=xsc+x2
                y=ysc+y2
                z=zsc+z2
                ro=np.sqrt(x**2+y**2+z**2)
                phio=np.arctan2(x,y)
                theo=np.arccos(z/np.sqrt(x**2+y**2+z**2))
    #            print( ["{0:0.2f}".format(i) for i in [x,y,z]])
                if ro<ri:
                    rate=9999
    #                print(x,y,z)
                    break
                los.append(np.array([x,y,z,ro]))
            los=np.array(los)
    #        omni=pd.concat([omni]*np.shape(xyz)[0])

            rm=R.from_euler('z',-4,degrees=True)
            xyz_np=(rm.as_matrix()@los[:,:-1].T).T
    #        xyz_np=los[:,:-1]

    #        print('Finding MP nodes')
            mpd=np.array(mpdf(r0,alpha,xyz_np))

    #       print('Finding BS nodes')
            #print(xyz_np)
            pts=appendSpherical_np(xyz_np)
            if model!='jer':
                bsd=np.array(bsdf_Jelinek(a_14,a_44,xyz_np))
            else:
                bsd=np.array(bsdf(D,f,pts))

            f_msh=(pts[:,3]-mpd)/(bsd-mpd)
            pts=np.hstack((pts,f_msh.reshape(pts.shape[0],1)))
        #    print(f_msh)

            input=pts[:,4:]
    #        print(np.shape(input))

            #print(phi,np.shape(nfl),np.shape(n1),np.shape(n2),np.shape(input),np.shape(f),np.shape(f2))
            n=genf(nfl,n1,n2,input,f1,f2)
            T=genf(Tfl,n1,n2,input,f1,f2)
            Vx=genf(Vxfl,n1,n2,input,f1,f2)
            Vy=genf(Vyfl,n1,n2,input,f1,f2)
            Vz=genf(Vzfl,n1,n2,input,f1,f2)
    #        V_0=np.vstack([Vx,Vy,Vz]).T
    #        V=(rm.as_matrix().T@V_0.T).T
            vth=np.sqrt(3*1.38e-23*(T*11600)/1.6727e-27)*100
            vbu=np.sqrt(Vx**2+Vy**2+Vz**2)*1e5
            vrel=np.sqrt(vth**2+vbu**2)
            nn=nh0*(10./los[:,-1])**3
            xray=n*vrel*nn*drad*100*crs #eV/cm^3/s
            rate=np.nansum(xray/str1) #cV/cm^2/s/sr
    #        np.set_printoptions(precision=3,suppress=True,threshold=sys.maxsize,linewidth=150)
    #        print(np.shape(xyz_np[:,1]),np.shape(n),np.shape(vrel),np.shape(nn),np.shape(xray))
            sh=np.shape(xyz_np)[0]

        #    index=~np.isnan(n)
        #    print(len(index))

        #    test=np.hstack((pts[:,0:4],f_msh.reshape(sh,1),n.reshape(sh,1),vrel.reshape(sh,1),nn.reshape(sh,1),xray.reshape(sh,1),mpd.reshape(sh,1),bsd.reshape(sh,1)))
            #print(test[index])
    #        fo2.write(str(test[index]))
    #        fo2.write('\n\n')
        #    np.savetxt('test.txt',test[index],fmt='%.3f')
        #    np.savetxt('test_0.txt',test,fmt='%.3f')

            fo.write('%d %d %f %f %f %e\n' % (j,i,r,phi,lat,rate))
    #        print(phi,lat,rate)
        print(lat,'/',90-the2)
    fo.close()
    #fo2.close()

if __name__ == "__main__":
    import sys
    # If A.py is executed directly, call main_function with command-line arguments
    main(*sys.argv[1:])


import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
from scipy import stats
import sys

n=6
lf=int(sys.argv[1])
B=sys.argv[2]

run='Msh_Nstep'
external='easystore'

df_list2=[]
for i in range(1,n+1):
    df_list=[]
    for phi in np.arange(-90+7.5,90,7.5):
        a=pd.read_csv('/Volumes/'+external+'/openggcm_run/'+run+'/Spreiter/comp_'+B+'_'+str(1200+1200*i).zfill(6))
        if lf==1:a=a[(a.f>=.3) & (a.f<=.7)]
        df_list.append(a)
    a2=pd.concat(df_list)
    a2['n']=i*5
    df_list2.append(a2)
a3=pd.concat(df_list2)

stat=pd.DataFrame([[round(a3.ni.mean(),3),round(a3.V.mean(),3)],
[round(a3.ni_T.mean(),3),round(a3.V_T.mean(),3)],
[a3.ni.median(),a3.V.median()],
[a3.ni_T.median(),a3.V_T.median()]])
if lf!=1:stat.to_csv('/Volumes/'+external+'/openggcm_run/'+run+'/Spreiter/stat_%s_nv.txt' % B)
else:stat.to_csv('/Volumes/'+external+'/openggcm_run/'+run+'/Spreiter/stat_lf_%s_nv.txt' % B)


list=['ni','V']
unit=['[cm$^{-3}$]','[km/s]']
reg=[]
for j in range(2):
    mask=~np.isnan(a3[list[j]+'_T'])&~np.isnan(a3[list[j]])
    reg.append(stats.linregress(a3[list[j]+'_T'][mask],a3[list[j]][mask]))

label_l=np.arange(5,35,5)
fig,ax=plt.subplots(1,2,figsize=(10,5))
for j in range(2):
    ax_t=ax.flatten()[j]
    for i in range(n):
        ax_t.scatter(df_list2[i][list[j]+'_T'],df_list2[i][list[j]],label=str(label_l[i])+'cm$^{-3}$')
    ax_t.plot(a3[list[j]+'_T'],reg[j].slope*a3[list[j]+'_T']+reg[j].intercept)
    ax_t.annotate('y={:.2f}x+{:.2f}'.format(reg[j].slope,reg[j].intercept),xy=(.05,.95),xycoords='axes fraction')
    ax_t.plot(a3[list[j]+'_T'],a3[list[j]+'_T'],label='y=x')
    ax_t.grid()
#    ax_t.legend()
    ax_t.set_xlabel(list[j]+'_THEMIS '+unit[j])
    ax_t.set_ylabel(list[j]+'_Spreiter '+unit[j])
handles, labels = ax_t.get_legend_handles_labels()
fig.legend(handles, labels, loc='upper right',bbox_to_anchor=(0.9,0.99))
#plt.suptitle('B_z=-4nT, 0.3$\leq$f$\leq$0.7')
txt='Spreiter'
if B=='cn':txt+=' B>0'
else:txt+=' B<0'
if lf==1:txt+='\n 0.3$\leq$f$\leq$0.7'
plt.suptitle(txt)

path='/Volumes/'+external+'/openggcm_run/'+run+'/Spreiter/Scatter'
if lf==1: path+='_lf'
if B=='cn':path+='_cn'
else:path+='_cs'
path+='_nv.png'
plt.savefig(path)
